/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
#include "pythiaPlugin.h"

using namespace std;
using namespace Moses1;


int main() {
	cout << "Holy Moses..." << endl;
	
	// Generator. Process selection. LHC initialization. Histogram.
	Pythia pythia;

    //------------------------------------------------------
    // Chosen new PDF set; LHAPDF file name conventions.
    string pdfSet = "MRST2001lo.LHgrid";
    string pdfMember_str = "0";
    //------------------------------------------------------
	// PDF initialization
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	pythia.readString("PDF:LHAPDFmember = " + pdfMember_str);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 10");
	//------------------------------------------------------


	//------------------------------------------------------
	int outFermionID = 6; // top
	int nKKmodes = 1; // number of KK modes

	// A class to generate the process from external matrix element. 
	SigmaProcess* myProcess = new FGB(outFermionID, nKKmodes, pdfSet, 0);	

	// Hand pointer to Pythia.
	pythia.setSigmaPtr(myProcess);
	cout << "Hand pointer to Pythia" << endl;

	// id:all = name antiName spinType chargeType colType m0 mWidth mMin mMax tau0
	pythia.readString("500021:all = gluonAvi void 3 0 0 3000. 120. 10.00000 0.00000");
	pythia.readString("500021:isResonance = false");
	
	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 1000");
	pythia.readString("PhaseSpace:mHatMax = 5000");

	// Initialization for LHC.
	pythia.init(2212, 2212, 14000.);
	cout << "Initialization for LHC" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();
	
	commonOperations commonOps;
	string dirName = "/";	

	string filePath_mosesEventrecord = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"mosesFGBrunRecord.root");
	TFile *mosesEventRecordFile = new TFile(filePath_mosesEventrecord.c_str(),"recreate");	

	// chose what particle types to record in the record file
	//string record[] = {"muon", "electron", "photon", "neutrino", "meson", "baryon"};
	// build a vector of strings from this array
	//vector<string> sParticleRecords;
	//copy( record, record + sizeof(record)/sizeof(*record), inserter(sParticleRecords, sParticleRecords.end()) );

	// chose what particle types to record in the record file
	vector<string> sParticleRecords;
	sParticleRecords.push_back("muon");
	sParticleRecords.push_back("electron");
	sParticleRecords.push_back("photon");
	sParticleRecords.push_back("neutrino");
	sParticleRecords.push_back("meson");
	sParticleRecords.push_back("baryon");
	
	// create the mosesEvent instance (control everything in the run record)
	mosesEvent mosesevent(mosesEventRecordFile, sParticleRecords);
	
	// chose what jet types to record in the record file (if at all)
	// cluster jet (can be NULL if not needed):
	mosesClusterJetWrap* clusterJetWrap = new mosesClusterJetWrap("Lund", 2, 2, false, false);
	clusterJetWrap->setClusterJetAnalyzer(0., 50., 1, 0);
	// cell jet (can be NULL if not needed):
	mosesCellJetWrap* cellJetWrap = new mosesCellJetWrap(5., 50, 32, 2, 0, 0.5, 2., 0., 0);
	cellJetWrap->setCellJetAnalyzer(20., 0.7, 1.5);
	// fast jet (can be NULL if not needed):
	mosesFastJetWrap* fastJetWrap = new mosesFastJetWrap();
	fastJetWrap->setFastJetAnalyzer(0.4, fastjet::Best, fastjet::E_scheme, fastjet::kt_algorithm);
	
	// tell mosesEvent to record also the jet(s)
	// can delete the next line or pass 3 NULL arguments instead, if jets are not needed
	mosesevent.passJetFinderWrappers(clusterJetWrap, cellJetWrap, fastJetWrap); // each one can be NULL if not needed
	
	// write the set name in the event record header (not mandatory...)
	mosesevent.giveLHAPDFset(pdfSet);
	
	// run
	int nEvents  = 10000;
	cout << "Events =? ";
	cin >> nEvents;
	int printMod = (int)nEvents/10;
	mosesevent.loopOverEvents(nEvents, printMod, pythia);
	
	pythia.statistics();
	mosesEventRecordFile->Close();
	
	// Done.
	return 0;
}
