/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
//-----------------
#include "pythiaPlugin.h"

using namespace std;
using namespace Pythia8;

namespace Moses1 {

// A derived class for a completely new, user defined (from scratch) process, given as an input to pythia

void FGB::initProc() {

	// initialize parameters
	m_prm.setParameters(m_physPrm); // initialize the new instance of the physical parameters struct
	m_prm.setParameters(m_runPrm); // initialize the new instance of the run parameters struct
	m_prm.printUnits(); // just a reminder about units...

	// initialize
	m_pdtPrt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// fill all quarks and leptons properties in a single vector:
	m_pdtPrt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), m_PDTfermions, false);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	m_pdtPrt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), m_PDTbosons, false); // fill all PDTbosons properties in a single vector

	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	m_customPrt.setCUSTOMparticle(m_PDTfermions, m_CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here

	// initialize lha (lhapdf instance)
	//string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq6.LHpdf";
	//string pdfSet = "cteq5l.LHgrid";
	//string pdfSet = "MRST2006nnlo.LHgrid";
	string pdfSet = m_pdfSet;
	int subset = m_pdfMember;
	m_lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name

	// the selection of the outgoing fermion is done in the main program where m_idNew is being initialized
	for (int i=0 ; i<(int)m_CUSTOMfermions.size() ; i++) {
		if ( m_CUSTOMfermions[i].pdt_prt.id == (int)abs(m_idNew) ) { 
			m_outFermion1 = m_CUSTOMfermions[i];
			m_outFermion2 = m_CUSTOMfermions[i]; // S-channel
			
		}
	}
	
	
	cout << "outFermions are: " << m_outFermion1.pdt_prt.name << " and its anti" << endl;
	// the selection of the number of KK modes is done in the main program where m_nKKmodes is being initialized
	cout << "number of KK modes is: " << m_nKKmodes << endl;

	// Secondary open width fraction, relevant for top (or heavier).
	m_openFracPair = particleDataPtr->resOpenFrac(m_idNew, -m_idNew);

	cout << "initProc()...OK" << endl;
}


// Evaluate sigmaKin(sHat); first step when inflavours unknown.
void FGB::sigmaKin() {

	// Check that above threshold.
	m_isPhysical = true;
	// m3 m4 comes from sigmaProcecs
	if (mH < m3 + m4 + MASSMARGIN) {
		m_isPhysical = false;
		return;
	}
	// Define average F, Fbar mass so same beta. Phase space.
	double s34Avg  = 0.5 * (s3 + s4) - 0.25 * pow2(s3 - s4) / sH;
	m_mr             = s34Avg / sH;
	m_betaf          = sqrtpos(1. - 4. * m_mr);

	// Reconstruct decay angle so can reuse 2 -> 1 cross section.
	m_cosThe         = (tH - uH) / (m_betaf * sH);
/////////////////////////up to here dont touch/////////////////////////////////////////
////////////////////////////noam's code/////////////////////////////////////////////////
//	m_totalwidth_Z0 = 0;
//	m_totalwidth_gammaKK = 0;
//	m_totalwidth_ZKK = 0;
//	// calculate the total width of Z0 for each s_hat:
//	for (int i=0 ; i<(int)m_CUSTOMfermions.size() ; i++) {
//		switch (m_CUSTOMfermions[i].pdt_prt.id) {
//			case PDTNUE: break;
//			case PDTNUMU: break;
//			case PDTNUTAU: break;
//			default: // gamma can not decay into neutrino pair (while Z can)
//				m_totalwidth_gammaKK += m_wdt.getpartialWidth(1,sH,m_PDTbosons[1],m_CUSTOMfermions[i],m_physPrm); // gammaStar width
//				break;
//		}
//		m_totalwidth_ZKK += m_wdt.getpartialWidth(1, sH, m_PDTbosons[2], m_CUSTOMfermions[i], m_physPrm); // Z* total width
//		m_totalwidth_Z0 += m_wdt.getpartialWidth(0, sH, m_PDTbosons[2], m_CUSTOMfermions[i], m_physPrm); // Z0 total width
//	}
//////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
//in the hedder double m_totalwidth_GKK[3][9]; //2 helcity states 8 lambda metrixces,6 quarks, ([2][0][6] = 0)
    
	double hOut1=-0.5 ;
	double hOut2=+0.5 ;
	int Group = 0;
	double tempS=-1.;   //  tempS=-1.  this ignors the kinematic condition.
	/////////////////////////////////////////////////////////////////////////////////
	// Avi -  total width calculation s-channel in_f = out_f
	////////////////////////////////////////////////////////////////////////////////
	m_cplng.setFGBqQbarCouplings(m_physPrm);
	for (int KKmode=1 ; KKmode<=m_nKKmodes ; KKmode++) { // loop over all KK modes .one KK mode only for this model
				for ( Group=0 ; Group<3 ; Group++){
					for (int Lambda_I=1 ; Lambda_I<9 ; Lambda_I++){ //running over gell mann metrixecs // every lambda needs a tota width
					   ////////////////////////////////////////////////////////////
					   //////////////////////////starting to sum per Massive - G/////////////////////////////
					   m_totalwidth_GKK[Group][Lambda_I] = 0;
					   //	cout << "Group ([" << Group << "Lambda_I" << Lambda_I << ":" << endl;
						switch  (Group){
						 		case 0:{
									hOut1=-0.5;
									hOut2=-0.5;

									for (int i=0 ; i<6; i++) 
									{ // running over all of out going q - qbar pairs = s-channel.
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										m_totalwidth_GKK[Group][Lambda_I] += m_wdt.getpartialMassiveGWidth(Lambda_I, KKmode, tempS,
																		 m_PDTbosons[0], m_CUSTOMfermions[i], m_CUSTOMfermions[i],																		hOut1,hOut2, m_physPrm, m_cplng);
									}//for
									break;
								}//case
								case 1:{
									hOut1=+0.5;
									hOut2=+0.5;
									for (int i=0 ; i<6; i++) 
									{ // running over all of out going q - qbar pairs = s-channel.
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										m_totalwidth_GKK[Group][Lambda_I] += m_wdt.getpartialMassiveGWidth(Lambda_I, KKmode, tempS,
																		  m_PDTbosons[0], m_CUSTOMfermions[i], m_CUSTOMfermions[i], 																		    hOut1, hOut2, m_physPrm, m_cplng);
										i++; // down type
									}//for
									break;
								}//case
								case 2:{
									hOut1=+0.5;
									hOut2=+0.5;
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
									for (int i=1 ; i<6; i++) 
									{ // running over all of out going q - qbar pairs = s-channel.
										m_totalwidth_GKK[Group][Lambda_I] += m_wdt.getpartialMassiveGWidth(Lambda_I, KKmode, tempS,
																		  m_PDTbosons[0], m_CUSTOMfermions[i], m_CUSTOMfermions[i], 																			hOut1, hOut2, m_physPrm, m_cplng);
 										i++; // up type
									}//for
									break;
								}//case
							}//switch
					       //	cout << "Group - " << Group << " Lambda_I" << Lambda_I << ":" << "total width is " << m_totalwidth_GKK[Group][Lambda_I] << " GeV" << endl;

					}//for Lambda_I
			  }//for (double Group
		}//for KKmode


}



// Evaluate sigmaHat(sHat); first step when inflavours unknown.
double FGB::sigmaHat() {

        // Fail if below threshold.
        if (!m_isPhysical) {cout << "pythiaPlugin FGB::sigmaHat not physical, mH=" << (int)mH <<" m3=" << (int)m3 << " m4=" << (int)m4 << endl; return 0.;}
        // Cross section vanishing except for incoming qqar.
        if (abs(id1) != abs(id2)) {cout << "id1!=-id2" << endl; return 0.;}
        // if not supported quarks
        if ( (int)abs(id1)<PDTDWN || (int)abs(id1)>PDTTOP ) {cout << "in FGB::sigmaHat:1 parton id=" << (int)abs(id1) << " is not supported" << endl; return 0.;}
        // if not supported final state 1
   //???avi     if ( (int)abs(m_idNew)<PDTE && (int)abs(m_idNew)>=PDTTOP ) {cout << "FGB::sigmaHat:2 final state id=" << (int)abs(m_idNew)
		//???avi						     << " is not supported" << endl; return 0.;}
        // if not supported final state 2
        if ( (int)abs(m_idNew)<PDTDWN || (int)abs(m_idNew)>PDTTOP ) {cout << "FGB::sigmaHat:3 final state id="
									    << (int)abs(m_idNew) << " is not supported" << endl; return 0.;}

        for (int i=0 ; i<(int)m_CUSTOMfermions.size() ; i++) {
                if ( m_CUSTOMfermions[i].pdt_prt.id == (int)abs(id1) ) {m_inFermion1 = m_CUSTOMfermions[i];}
				if ( m_CUSTOMfermions[i].pdt_prt.id == (int)abs(id2) ) {m_inFermion2 = m_CUSTOMfermions[i];}

        }
////////////////////m_idNew ???????? one ??????????????
        // initialize values
        m_sigma = 0;
		
        // calculate the averaged (over all helicity states) helicity square amplitude while summing all possible polarizations
		//differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
	    //differentialSquareAmp_KK = 0; // new kk only
		int Group = 0;
		int q = m_inFermion1.pdt_prt.id;
        for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
                for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

                        // get the angularDistribution (determined by helicity states) ???????????? Avi
                       // example noams code:
					     m_thetaDistribution = 0;
                         if (hIn==hOut) {m_thetaDistribution = (1+m_cosThe)*(1+m_cosThe);}
                         else {m_thetaDistribution = (1-m_cosThe)*(1-m_cosThe);}
					   /////////////////////////////////////////////////////////////////////////
					   
					   m_fullAmp = dcomplex(0,0);
						if (hIn==hOut) {
							if (hIn < 0.) {
								Group = 0; // left handed
							}//if
						 else if  (((q == 1)||(q==3)||(q==5))&&(hIn > 0.)) {
							Group = 1; //down type right handed ;
						 }//if
						 else if  (((q == 0)||(q==2)||(q==4))&&(hIn > 0.)){
							Group = 2; //up type right handed ;
						 }//if
						 for (int Lambda_I =1 ;Lambda_I <9;Lambda_I++){ //summing over all of the glouns in this group
							m_fullAmp += m_hAmp.gethelicityMassiveGAmplitude(Lambda_I, m_nKKmodes, sH, m_totalwidth_GKK[Group][Lambda_I],
																			hIn, hIn, hOut, hOut, m_inFermion1,m_inFermion1,m_outFermion1, 
																			m_outFermion2,m_PDTbosons[0],m_physPrm, m_cplng);
					 }//for
					}//if Hin== Hout


        		// calculate square amplitude for each helicity states combinations
			// ----------example ----------
			// ---------- SM terms ----------
			// photon propagator
		//	m_fullAmp += m_hAmp.gethelicityAmplitude(sH,0,hIn,hOut,m_inFermion1,m_outFermion1,m_PDTbosons[1],m_physPrm);
			// Z0 propagator
		//	m_fullAmp += m_hAmp.gethelicityAmplitude(sH,m_totalwidth_Z0,hIn,hOut,m_inFermion1,m_outFermion1,m_PDTbosons[2],m_physPrm);
			// ------------------------------


			m_fullAmp *= conj(m_fullAmp);
			m_sigma += (1./4.) * ( real(m_fullAmp) * m_thetaDistribution );
		} // end hIn
	} // end hOut

	// Top: corrections for closed decay channels. ?????????????????????????? What the hall
	m_sigma *= m_openFracPair;

	// Multiply the averaged squared helicity amplitude by some factors:
	// [1] 2/sHat is received from the transformation between dtHat and dcosTheta.
	//     A 2->1 process such as qqbar->gamma*/Z0 is only sigmaHat(sHat).
	//     If you choose to include the decay, using a 2->2 process (2->1->2),
	//     then you instead use d(sigmaHat)/d(tHat), which dimensionally is a factor
	//     1/sHat different. Althernatively you can think of it in terms of the phase
	//     space integral 1 = integral_{-sHat}{0} d{tHat)/sHat.
	//     tHat=(-sHat/2)*(1-cosTheta) ==> dtHat=(sHat/2)*dcosTheta ==> d/dtHat=(2/sHat)*d/dcosTheta.
	// [2] 2*pi is the integral over dphi
	// [3] (1/4sHat)*(Nc_f/Nc_q) is the normal outer factors out of the averaged matrix element
	// [4] sHat^2 is to fix the units and to cancel the 1/sHat^2 that is artificially included in the
	//     matrix element (for the photon bit it is inserted by hand, for the Z0 bit it is inherent)
	// [5] 1/(4 pi)^2 =  the effective coupeling has this factor eg g^2/4*pi .
	m_sigma *= (2./sH) * (2.*m_physPrm.pi)* 1./(pow2(4.*m_physPrm.pi))*1./(4.*sH) * (double)m_outFermion1.Nc/(double)m_inFermion1.Nc // use running alpha_em
		   * pow2(sH);

		// [m_sigma] = [1/GeV^4] --> [Integrate{m_sigma*ds}] = [1/GeV^2] --> OK !!!
		/*--------------------------------------------------------------------------------*/
		/* it is assumed that cross sections normally come in dimensions such that they,  */
		/* when integrated over the relevant phase space, obtain the dimension [1/GeV^2], */
		/* and therefore need to be converted to mb                                       */
		/* If the cross section is already encoded as mb then convert2mb() should be      */
		/* overloaded to instead return false                                             */
		/*--------------------------------------------------------------------------------*/

	// incoming color possibilities (PDFs returns 9 combinations, only 3 are valid for neutral current)
	if (abs(id1)<9) {m_sigma /= 3;}

	// Cross section as already calculated.
	return m_sigma;

}



// Select identity, colour and anticolour.
void FGB::setIdColAcol() {

	// Set outgoing flavours.
	//id3 = (id1 > 0) ? m_idNew : -m_idNew;
	id3 = (id1 > 0) ? abs(m_idNew) : -abs(m_idNew);
	setId( id1, id2, id3, -id3);

	//setId(id1, id2, idOut, -1*idOut); // id1,id2 is given by pythia

	// Colour flow topologies. Swap when antiquarks.
	if (id1<0) {setColAcol(0,1, 1,0, 0,2, 2,0);} // id1 is an anti-quark
	else {setColAcol(1,0, 0,1, 2,0, 0,2);} // id1 is a quark
}



}
