/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/";
	string outFileName = dirName + "Binbybin_kinematics";
	cout << "\n" << endl;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	bool doTrue = true;
        anlz.setTrue(doTrue);
	Particle particle;

	double minM = 200;
	double maxM = 1000;
	cout << "choose mHat min...";
	cin >> minM;
	cout << "choose mHat max...";
        cin >> maxM;
	
	runPrm.invMassCut = minM;

	// set cuts
	//double invMassCut=1000.;
	double invMassCut=minM;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	int cutSelector = 1;
	cout << "select cuts to apply: no cuts (-1) all cuts (0), only mHat cut (1), only pT cut (2), only eta cut (3)..." << endl;
	cin >> cutSelector;
	switch(cutSelector) {
		case -1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 invMassCut = 0.;
			 break;
		case  0: break;
		case  1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  2: invMassCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  3: invMassCut = 0.;
			 pTransverseCut = 0.;
			 break;
		default: cout << "wrong option cutSelector = " << cutSelector << "\t set to default cutSelector=0" << endl;
			 break;
	}
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);


	int sampleSizeSelector = 1;
	cout << "select input sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3), L=10fb^-1(4)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector<1 || sampleSizeSelector>4) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_LrefinvFb"; break;
		case 2: sSampleSize = "_L500invFb"; break;
		case 3: sSampleSize = "_L100invFb"; break;
		case 4: sSampleSize = "_L10invFb"; break;
		default: break;
	}
	outFileName += sSampleSize;

	
	int mStarSelector = 4;
	string sMStarSelector = "_mStar=4TeV";
	if(sampleSizeSelector==4) {
		cout << "select m^* value: 3TeV(3), 4TeV(4)...";
		cin >> mStarSelector;
		if (mStarSelector==3) {
			sMStarSelector = "_mStar=3TeV";
			physPrm.mStar = 3000.;
			physPrm.mZprime = 3000.;
			physPrm.wZprime = 80.;
		}
		else {
			sMStarSelector = "_mStar=4TeV";
			physPrm.mStar = 4000.;
                        physPrm.mZprime = 4000.;
                        physPrm.wZprime = 120.;
		}
	}
	outFileName += sMStarSelector;


	string binbybinRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".root");
	string binbybinEpsFileName_mHat = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mHat.eps");
	string binbybinEpsFileName_pT   = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_pT.eps");

	TFile *binbybinHistFile = new TFile(binbybinRootFileName.c_str(),"recreate");	
	TDirectory* dir_allHistograms = binbybinHistFile->mkdir("allHistograms");
	dir_allHistograms->cd();

	TDirectory* dir_binbybin_mHat = binbybinHistFile->mkdir("binbybin_mHat");
	dir_binbybin_mHat->cd();
	TCanvas *canv_binbybin_mHat = new TCanvas("canv_binbybin_mHat" ,"Bin by bin mHat",0,0,1400,800);
	canv_binbybin_mHat->Divide(1,2);
	TVirtualPad *pad_mHat=canv_binbybin_mHat->cd(1);
	pad_mHat->SetLogy();

	TDirectory* dir_mHat = dir_binbybin_mHat->mkdir("mHat");
        dir_mHat->cd();
	TCanvas *canv_mHat = new TCanvas("canv_mHat" ,"canv_mHat",0,0,1400,800);
        canv_mHat->SetLogy();

	TDirectory* dir_mHat_relDiff = dir_binbybin_mHat->mkdir("mHat_relDiff");
        dir_mHat_relDiff->cd();
        TCanvas *canv_mHat_relDiff = new TCanvas("canv_mHat_relDiff" ,"canv_mHat_relDiff",0,0,1400,800);



	TDirectory* dir_binbybin_pT = binbybinHistFile->mkdir("binbybin_pT");
	dir_binbybin_pT->cd();
	TCanvas *canv_binbybin_pT = new TCanvas("canv_binbybin_pT","Bin by bin pT",0,0,1400,800);
        canv_binbybin_pT->Divide(1,2);
        TVirtualPad *pad_pT=canv_binbybin_pT->cd(1);
        pad_pT->SetLogy();

	TDirectory* dir_pT = dir_binbybin_pT->mkdir("pT");
        dir_pT->cd();
        TCanvas *canv_pT = new TCanvas("canv_pT" ,"canv_pT",0,0,1400,800);
        canv_pT->SetLogy();

        TDirectory* dir_pT_relDiff = dir_binbybin_pT->mkdir("pT_relDiff");
        dir_pT_relDiff->cd();
        TCanvas *canv_pT_relDiff = new TCanvas("canv_pT_relDiff" ,"canv_pT_relDiff",0,0,1400,800);


	// boundries and bins of histograms
	Int_t invMnBins = 50;
	Double_t invMmin = invMassCut, invMmax = maxM;

	// choose mHat & pT binnings
	switch(sampleSizeSelector) {
		case 1:	invMnBins = 50; break; // 1M sample
		case 2: invMnBins = 14; break; // 500[fb^-1] sample
		case 3: invMnBins = 14; break; // 100[fb^-1] sample
		case 4: invMnBins = 19; break; // 10 fb^-1 sample
		default: break;
	}


	Int_t    pTnBins = 50; // 100
	Double_t pTmin   = 10.;
	Double_t pTmax   = 4000.;
	Double_t pTBoundToReduce = 1000.;
	Double_t invMassBoundToReduce = 1000.;
	if (sampleSizeSelector==2 || sampleSizeSelector==3) { // small samples
		pTnBins = 20;
	}
	else if (sampleSizeSelector==4)
	{
		pTmax =  1200;
		pTBoundToReduce = 0;
		pTnBins = 14;
		invMnBins = 19;
		invMassBoundToReduce = 0.;
	}


	if (cutSelector==-1 || cutSelector==1 || cutSelector==2 || cutSelector==3) {
		pTnBins = 100;
		pTmin = 5.;
		pTmax = 100.;
		
		invMnBins = 500;
		invMmin = 5.;
		invMmax = 6000.;
	}


	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax - invMassBoundToReduce);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }

	Double_t logpTmin = log10(pTmin);
	Double_t logpTmax = log10(pTmax - pTBoundToReduce);
	Double_t pT_binwidth = (Double_t)( (logpTmax-logpTmin)/(Double_t)pTnBins );
	Double_t pT_bins[pTnBins+1];
	pT_bins[0] = pTmin;
	for (Int_t i=1 ; i<=pTnBins ; i++) { pT_bins[i] = TMath::Power( 10,(logpTmin + i*pT_binwidth) ); }


	dir_allHistograms->cd();


	//----- SM -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *SM_invMass = new TH1D("SM_invMass", "", invMnBins, invMmin, invMmax);
	SM_invMass->SetLineStyle(5);
	SM_invMass->SetLineColor(kBlue);
	SM_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SM_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *SM_pT = new TH1D("SM_pT", "", pTnBins, pTmin, pTmax);
	SM_pT->SetLineStyle(5);
	SM_pT->SetLineColor(kBlue);
	SM_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	SM_pT->SetYTitle("Events");



	//----- KK -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *KK_invMass = new TH1D("KK_invMass", "", invMnBins, invMmin, invMmax);
	KK_invMass->SetLineStyle(1);
	KK_invMass->SetLineColor(kRed);
	KK_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *KK_pT = new TH1D("KK_pT", "", pTnBins, pTmin, pTmax);
	KK_pT->SetLineStyle(1);
	KK_pT->SetLineColor(kRed);
	KK_pT->SetXTitle("#p_{T}(#mu^{-})[GeV]");
	KK_pT->SetYTitle("Events");

	

	//----- Z' -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *Zprime_invMass = new TH1D("Zprime_invMass", "", invMnBins, invMmin, invMmax);
	Zprime_invMass->SetLineStyle(3);
	Zprime_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	Zprime_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *Zprime_pT = new TH1D("Zprime_pT", "", pTnBins, pTmin, pTmax);
	Zprime_pT->SetLineStyle(3);
	Zprime_pT->SetXTitle("#p_{T}(#mu^{-})[GeV]");
	Zprime_pT->SetYTitle("Events");


	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg_mHat = new TLegend(0.7,0.5,0.96,0.7);
	leg_mHat->AddEntry(SM_invMass,"SM","L");
	leg_mHat->AddEntry(KK_invMass,"KK","L");
	leg_mHat->AddEntry(Zprime_invMass,"Z'_{SM}","L");
	
	TLegend *leg_pT = new TLegend(0.7,0.7,0.96,0.9);
	leg_pT->AddEntry(SM_pT,"SM","L");
	leg_pT->AddEntry(KK_pT,"KK","L");
	leg_pT->AddEntry(Zprime_pT,"Z'_{SM}","L");


	// binbybin invariant mass histos
	dir_binbybin_mHat->cd(); 
	TH1D *SM_mHat_binbybin = new TH1D("SM_mHat_binbybin", "", invMnBins, invMmin, invMmax);
	TH1D *KK_mHat_binbybin = new TH1D("KK_mHat_binbybin", "", invMnBins, invMmin, invMmax);
	TH1D *Zprime_mHat_binbybin = new TH1D("Zprime_mHat_binbybin", "", invMnBins, invMmin, invMmax);
	
	dir_mHat_relDiff->cd();
	TH1D *KK_positive_diff_mHat = new TH1D("KK_positive_diff_mHat", "", invMnBins, invMmin, invMmax);
	KK_positive_diff_mHat->SetFillStyle(3354);
	KK_positive_diff_mHat->SetFillColor(kRed);
	KK_positive_diff_mHat->SetLineColor(kRed);
        KK_positive_diff_mHat->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
        KK_positive_diff_mHat->SetYTitle("relative difference, D");

        TH1D *KK_negative_diff_mHat = new TH1D("KK_negative_diff_mHat", "", invMnBins, invMmin, invMmax);
	KK_negative_diff_mHat->SetFillColor(kBlack);
	KK_negative_diff_mHat->SetFillStyle(3354);
	KK_negative_diff_mHat->SetFillColor(kRed);
        KK_negative_diff_mHat->SetLineColor(kRed);
	KK_negative_diff_mHat->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
	KK_negative_diff_mHat->SetYTitle("relative difference, D");

	TH1D *Zprime_positive_diff_mHat = new TH1D("Zprime_positive_diff_mHat", "", invMnBins, invMmin, invMmax);
	Zprime_positive_diff_mHat->SetFillColor(kBlack);
	Zprime_positive_diff_mHat->SetFillStyle(3005);
	Zprime_positive_diff_mHat->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
	Zprime_positive_diff_mHat->SetYTitle("relative difference, D");

	TH1D *Zprime_negative_diff_mHat = new TH1D("Zprime_negative_diff_mHat", "", invMnBins, invMmin, invMmax);
	Zprime_negative_diff_mHat->SetFillColor(kBlack);
	Zprime_negative_diff_mHat->SetFillStyle(3005);
	Zprime_negative_diff_mHat->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
	Zprime_negative_diff_mHat->SetYTitle("relative difference, D");
	
	TLine *zeroLine_mHat = new TLine(invMmin,0.,invMmax,0.);

	TLegend *leg_binbybin_mHat = new TLegend(0.7,0.7,0.96,0.9);
        leg_binbybin_mHat->AddEntry(KK_positive_diff_mHat,"KK", "f");
        leg_binbybin_mHat->AddEntry(Zprime_positive_diff_mHat,"Z'_{SM}", "f");



	// binbybin pT histos 
	dir_binbybin_pT->cd();
        TH1D *SM_pT_binbybin = new TH1D("SM_pT_binbybin", "", pTnBins, pTmin, pTmax);
        TH1D *KK_pT_binbybin = new TH1D("KK_pT_binbybin", "", pTnBins, pTmin, pTmax);
        TH1D *Zprime_pT_binbybin = new TH1D("Zprime_pT_binbybin", "", pTnBins, pTmin, pTmax);

	dir_pT_relDiff->cd();
        TH1D *KK_positive_diff_pT = new TH1D("KK_positive_diff_pT", "", pTnBins, pTmin, pTmax);
        KK_positive_diff_pT->SetFillStyle(3354);
        KK_positive_diff_pT->SetFillColor(kRed);
        KK_positive_diff_pT->SetLineColor(kRed);
        KK_positive_diff_pT->SetXTitle("p_{T}(#mu^{-}) [GeV]");
        KK_positive_diff_pT->SetYTitle("relative difference, D");
        
        TH1D *KK_negative_diff_pT = new TH1D("KK_negative_diff_pT", "", pTnBins, pTmin, pTmax);
        KK_negative_diff_pT->SetFillColor(kBlack);
        KK_negative_diff_pT->SetFillStyle(3354);
        KK_negative_diff_pT->SetFillColor(kRed);
        KK_negative_diff_pT->SetLineColor(kRed);
        KK_negative_diff_pT->SetXTitle("p_{T}(#mu^{-}) [GeV]");
	KK_negative_diff_pT->SetYTitle("relative difference, D");

        TH1D *Zprime_positive_diff_pT = new TH1D("Zprime_positive_diff_pT", "", pTnBins, pTmin, pTmax);
        Zprime_positive_diff_pT->SetFillColor(kBlack);
        Zprime_positive_diff_pT->SetFillStyle(3005);
	Zprime_positive_diff_pT->SetXTitle("p_{T}(#mu^{-}) [GeV]");
	Zprime_positive_diff_pT->SetYTitle("relative difference, D");
        
        TH1D *Zprime_negative_diff_pT = new TH1D("Zprime_negative_diff_pT", "", pTnBins, pTmin, pTmax);
        Zprime_negative_diff_pT->SetFillColor(kBlack);
        Zprime_negative_diff_pT->SetFillStyle(3005);
	Zprime_negative_diff_pT->SetXTitle("p_{T}(#mu^{-}) [GeV]");
	Zprime_negative_diff_pT->SetYTitle("relative difference, D");

        TLine *zeroLine_pT = new TLine(pTmin,0.,pTmax,0.);

        TLegend *leg_binbybin_pT = new TLegend(0.7,0.7,0.96,0.9);
        leg_binbybin_pT->AddEntry(KK_positive_diff_pT,"KK", "f");
        leg_binbybin_pT->AddEntry(Zprime_positive_diff_pT,"Z'_{SM}", "f");
	//----------------------------------------------------------------------------------------------------------


	// make the unbinned test:
	vector<Double_t> vec_SM_mHat;
	vector<Double_t> vec_SM_pT;
	vector<Double_t> vec_Zprime_mHat;
        vector<Double_t> vec_Zprime_pT;
	vector<Double_t> vec_KK_mHat;
        vector<Double_t> vec_KK_pT;


	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double invariantMass, pTransverse;

	// to count the number of events in the KK peak area
	double KKpeak_highLimit = 5000.;
	double KKpeak_lowLimit = 2000.;
	if (sampleSizeSelector==4) {
		KKpeak_highLimit = 2000.;
		KKpeak_lowLimit = 200.;
	}
	

	//string treeType = "testTree";
	string treeType = "testTree_allPairs";

	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=3 ; f++) {
		//-------------------------------------------------------------------------------------------------------------------
		// read the trees and put all good lepton pairs into 2 vectors
		//----- 1M
		if (sampleSizeSelector == 1) { //1M smaples
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_extractedParticles_2trees_1Msample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles_2trees_1Msample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_extractedParticles_2trees_1Msample.root");
			}
		}

		//----- 500
		else if (sampleSizeSelector == 2) { // 500[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
                        else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles_2trees_500inversefemtobarnSample.root");

                        }
                        else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
		}

		//----- 100
		else if (sampleSizeSelector == 3) { // 100[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
		}
		else if (sampleSizeSelector == 4) { // 10[fb^-1]
		 	if (f==1) { // SM
				if (mStarSelector==3) {
		 			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
			else if (f==2) { // KK
				if (mStarSelector==3) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
			else if (f==3) { // E6
				if (mStarSelector==3) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root");
				}
				else if(mStarSelector==4) {
					filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root");
				}
			}
		}




		Nevents = anlz.loopOverEvents(filePath, treeType.c_str(), 0,0);
		cout << "Nevents = " << Nevents << endl;
		// get all the good pairs (after cuts)
		Nleptons = anlz.getAllLeptons(allgoodLeptons);
		Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
		//-------------------------------------------------------------------------------------------------------------------
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			pTransverse = allgoodLeptons[k].pT;
			// fill the histos
			if (f==1) {
				SM_invMass->Fill( (Double_t)invariantMass );
				SM_mHat_binbybin->Fill( (Double_t)invariantMass );
				SM_pT->Fill( (Double_t)pTransverse );
				SM_pT_binbybin->Fill( (Double_t)pTransverse );

				vec_SM_mHat.push_back(invariantMass);
				vec_SM_pT.push_back(pTransverse);
			}
			else if (f==2) {
				KK_invMass->Fill( (Double_t)invariantMass );
				KK_mHat_binbybin->Fill( (Double_t)invariantMass );
				KK_pT->Fill( (Double_t)pTransverse );
				KK_pT_binbybin->Fill( (Double_t)pTransverse );
				
				vec_KK_mHat.push_back(invariantMass);
                                vec_KK_pT.push_back(pTransverse);
			}
			else if (f==3) {
				Zprime_invMass->Fill( (Double_t)invariantMass );
				Zprime_mHat_binbybin->Fill( (Double_t)invariantMass );
				Zprime_pT->Fill( (Double_t)pTransverse );
				Zprime_pT_binbybin->Fill( (Double_t)pTransverse );

				vec_Zprime_mHat.push_back(invariantMass);
                                vec_Zprime_pT.push_back(pTransverse);
			}
		}
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}


	cout << "\n" << endl;


	// resolve the relative differences (in each bin) between the internal and the external invariant mass data
	commonOps.relDiff((TH1D*)KK_mHat_binbybin, (TH1D*)SM_mHat_binbybin, (TH1D*)KK_positive_diff_mHat, (TH1D*)KK_negative_diff_mHat);
	commonOps.relDiff((TH1D*)Zprime_mHat_binbybin, (TH1D*)SM_mHat_binbybin, (TH1D*)Zprime_positive_diff_mHat, (TH1D*)Zprime_negative_diff_mHat);
	commonOps.relDiff((TH1D*)KK_pT_binbybin, (TH1D*)SM_pT_binbybin, (TH1D*)KK_positive_diff_pT, (TH1D*)KK_negative_diff_pT);
        commonOps.relDiff((TH1D*)Zprime_pT_binbybin, (TH1D*)SM_pT_binbybin, (TH1D*)Zprime_positive_diff_pT, (TH1D*)Zprime_negative_diff_pT);
	/*
	Double_t nBSM, nSM, reldiff;
	for (Int_t n=1 ; n<=invMnBins ; n++) {
		nSM = SM_mHat_binbybin->GetBinContent(n);
		nBSM = KK_mHat_binbybin->GetBinContent(n);
		if ((nSM+nBSM)!=0) { // for not dividing by zero...
			reldiff = (nSM-nBSM) / sqrt(nSM + nBSM);
			if (reldiff < 0) { KK_negative_diff_mHat->SetBinContent(n,reldiff); }
			else if (reldiff > 0) { KK_positive_diff_mHat->SetBinContent(n,reldiff); }
			else if (reldiff == 0) {
				KK_positive_diff_mHat->SetBinContent(n,0);
				KK_negative_diff_mHat->SetBinContent(n,0);
			}
		}
		else {
			KK_positive_diff_mHat->SetBinContent(n,0);
			KK_negative_diff_mHat->SetBinContent(n,0);
		}
	}
	*/
	/*	
	for (Int_t n=1 ; n<=invMnBins ; n++) {
		nSM = SM_mHat_binbybin->GetBinContent(n);
		nBSM = Zprime_mHat_binbybin->GetBinContent(n);
		if ((nSM+nBSM)!=0) { // for not dividing by zero...
			reldiff = (nSM-nBSM) / sqrt(nSM + nBSM);
			if (reldiff < 0) { Zprime_negative_diff_mHat->SetBinContent(n,reldiff); }
			else if (reldiff > 0) { Zprime_positive_diff_mHat->SetBinContent(n,reldiff); }
			else if (reldiff == 0) {
				Zprime_positive_diff_mHat->SetBinContent(n,0);
				Zprime_negative_diff_mHat->SetBinContent(n,0);
			}
		}
		else {
			Zprime_positive_diff_mHat->SetBinContent(n,0);
			Zprime_negative_diff_mHat->SetBinContent(n,0);
		}
	}
	*/

	 // chi2 and kolmogorov tests
	Double_t chi2       = KK_mHat_binbybin->Chi2Test(SM_mHat_binbybin, "UUCHI2/NDF");
	Double_t kolmogorov = KK_mHat_binbybin->KolmogorovTest(SM_mHat_binbybin, "N");
	cout << "KK mHat histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;

	chi2       = KK_pT_binbybin->Chi2Test(SM_pT_binbybin, "UUCHI2/NDF");
        kolmogorov = KK_pT_binbybin->KolmogorovTest(SM_pT_binbybin, "N");
        cout << "KK pT histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;

	chi2       = Zprime_mHat_binbybin->Chi2Test(SM_mHat_binbybin, "UUCHI2/NDF");
	kolmogorov = Zprime_mHat_binbybin->KolmogorovTest(SM_mHat_binbybin, "N");
	cout << "Z' mHat histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;

	chi2       = Zprime_pT_binbybin->Chi2Test(SM_pT_binbybin, "UUCHI2/NDF");
        kolmogorov = Zprime_pT_binbybin->KolmogorovTest(SM_pT_binbybin, "N");
        cout << "Z' pT histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
	

	
	// make the unbinned test
	Int_t SM_Entries_mHat = (Int_t)vec_SM_mHat.size();
        Int_t SM_Entries_pT = (Int_t)vec_SM_pT.size();
        Double_t *array_SM_mHat = (Double_t *)malloc(SM_Entries_mHat*sizeof(Double_t));
        Double_t *array_SM_pT = (Double_t *)malloc(SM_Entries_pT*sizeof(Double_t));
	// sort the vectors:
	sort(vec_SM_mHat.begin(), vec_SM_mHat.end());
        sort(vec_SM_pT.begin(), vec_SM_pT.end());
        // fill the sorted arrays:
        for (int j=0 ; j<(int)vec_SM_mHat.size() ; ++j) { array_SM_mHat[j] = vec_SM_mHat[j]; }
        for (int j=0 ; j<(int)vec_SM_pT.size() ; ++j) { array_SM_pT[j] = vec_SM_pT[j]; }

	Int_t KK_Entries_mHat = (Int_t)vec_KK_mHat.size();
        Int_t KK_Entries_pT = (Int_t)vec_KK_pT.size();
        Double_t *array_KK_mHat = (Double_t *)malloc(KK_Entries_mHat*sizeof(Double_t));
        Double_t *array_KK_pT = (Double_t *)malloc(KK_Entries_pT*sizeof(Double_t));
	// sort the vectors:
        sort(vec_KK_mHat.begin(), vec_KK_mHat.end());
        sort(vec_KK_pT.begin(), vec_KK_pT.end());
        // fill the sorted arrays:
        for (int j=0 ; j<(int)vec_KK_mHat.size() ; ++j) { array_KK_mHat[j] = vec_KK_mHat[j]; }
        for (int j=0 ; j<(int)vec_KK_pT.size() ; ++j) { array_KK_pT[j] = vec_KK_pT[j]; }

	Int_t Zprime_Entries_mHat = (Int_t)vec_Zprime_mHat.size();
        Int_t Zprime_Entries_pT = (Int_t)vec_Zprime_pT.size();
        Double_t *array_Zprime_mHat = (Double_t *)malloc(Zprime_Entries_mHat*sizeof(Double_t));
        Double_t *array_Zprime_pT = (Double_t *)malloc(Zprime_Entries_pT*sizeof(Double_t));
	// sort the vectors:
        sort(vec_Zprime_mHat.begin(), vec_Zprime_mHat.end());
        sort(vec_Zprime_pT.begin(), vec_Zprime_pT.end());
        // fill the sorted arrays:
        for (int j=0 ; j<(int)vec_Zprime_mHat.size() ; ++j) { array_Zprime_mHat[j] = vec_Zprime_mHat[j]; }
        for (int j=0 ; j<(int)vec_Zprime_pT.size() ; ++j) { array_Zprime_pT[j] = vec_Zprime_pT[j]; }


	// do the unbinned ks test:
	Double_t unbinnedKStest_KK_mHat = TMath::KolmogorovTest((Int_t)vec_SM_mHat.size(), array_SM_mHat, (Int_t)vec_KK_mHat.size(), array_KK_mHat ,"");
        Double_t unbinnedKStest_Zprime_mHat = TMath::KolmogorovTest((Int_t)vec_SM_mHat.size(), array_SM_mHat, (Int_t)vec_Zprime_mHat.size(), array_Zprime_mHat ,"");

        Double_t unbinnedKStest_KK_pT = TMath::KolmogorovTest((Int_t)vec_SM_pT.size(), array_SM_pT, (Int_t)vec_KK_pT.size(), array_KK_pT ,"");
	Double_t unbinnedKStest_Zprime_pT = TMath::KolmogorovTest((Int_t)vec_SM_pT.size(), array_SM_pT, (Int_t)vec_Zprime_pT.size(), array_Zprime_pT ,"");

        cout << endl;
	cout << "KK: mHat unbinned KStest = " << unbinnedKStest_KK_mHat << "\t" << "pT unbinned KStest = " << unbinnedKStest_KK_pT << endl;
        cout << "Z': mHat unbinned KStest = " << unbinnedKStest_Zprime_mHat << "\t" << "pT unbinned KStest = " << unbinnedKStest_Zprime_pT << endl;


	// scale reference histos
	// for 1M KK events, if the SM sample size is also 1M then the histos should be scaled by 1.2
	// for 1M KK events, if the E6 sample size is also 1M then the histos should be scaled by 1.15
	if (sampleSizeSelector==1) {
		SM_invMass->Scale(1.2);
		SM_pT->Scale(1.2);
		Zprime_invMass->Scale(1.15);
		Zprime_pT->Scale(1.15);
	}

	Double_t max1, max2, max;

	// bin by bin mHat histod
	dir_binbybin_mHat->cd();
	canv_binbybin_mHat->Draw();
	canv_binbybin_mHat->cd();
	canv_binbybin_mHat->cd(1);
	SM_invMass->SetMinimum(2.e-1);
	SM_invMass->Draw();
	Zprime_invMass->Draw("sames");
	KK_invMass->Draw("sames");
	leg_mHat->Draw("sames");
	
	canv_binbybin_mHat->cd(2);
        max1 = abs(KK_positive_diff_mHat->GetMinimum());
        max2 = abs(KK_positive_diff_mHat->GetMaximum());
        max = (max1 > max2) ? max1 : max2; // the ternary operator ( ? : )
        KK_positive_diff_mHat->SetMinimum(-2.*max);
        KK_positive_diff_mHat->SetMaximum(+2.*max);

	KK_positive_diff_mHat->Draw();
        KK_negative_diff_mHat->Draw("sames");
	Zprime_positive_diff_mHat->Draw("sames");
	Zprime_negative_diff_mHat->Draw("sames");
	zeroLine_mHat->Draw("sames");
	leg_binbybin_mHat->Draw("sames");
	
        canv_binbybin_mHat->Update();
        canv_binbybin_mHat->Write();



	dir_mHat->cd();
	canv_mHat->Draw();
        canv_mHat->cd();
        SM_invMass->Draw();
        Zprime_invMass->Draw("sames");
        KK_invMass->Draw("sames");
        leg_mHat->Draw("sames");
	canv_mHat->Update();
        canv_mHat->Write();

	dir_mHat_relDiff->cd();
	canv_mHat_relDiff->Draw();
	canv_mHat_relDiff->cd();
	KK_positive_diff_mHat->Draw();
        KK_negative_diff_mHat->Draw("sames");
        Zprime_positive_diff_mHat->Draw("sames");
        Zprime_negative_diff_mHat->Draw("sames");
        zeroLine_mHat->Draw("sames");
        leg_binbybin_mHat->Draw("sames");
	canv_mHat_relDiff->Update();
        canv_mHat_relDiff->Write();



	// bin by bin pT histod
       	dir_binbybin_pT->cd();
        canv_binbybin_pT->Draw();
        canv_binbybin_pT->cd();
        canv_binbybin_pT->cd(1);
        SM_pT->Draw();
	SM_pT->SetMinimum(2.e-1);
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
        leg_pT->Draw("sames");

        canv_binbybin_pT->cd(2);
        max1 = abs(KK_positive_diff_pT->GetMinimum());
        max2 = abs(KK_positive_diff_pT->GetMaximum());
        max = (max1 > max2) ? max1 : max2; // the ternary operator ( ? : )
        KK_positive_diff_pT->SetMinimum(-2.*max);
        KK_positive_diff_pT->SetMaximum(+2.*max);

        KK_positive_diff_pT->Draw();
        KK_negative_diff_pT->Draw("sames");
        Zprime_positive_diff_pT->Draw("sames");
        Zprime_negative_diff_pT->Draw("sames");
        zeroLine_pT->Draw("sames");
        leg_binbybin_pT->Draw("sames");

        canv_binbybin_pT->Update();
        canv_binbybin_pT->Write();
/*
	binbybinHistFile->cd("pT");
	canv_binbybin_pT->Draw();
	canv_binbybin_pT->cd();
        canv_binbybin_pT->cd(1);
        SM_pT->Draw();
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
        leg_mHat->Draw("sames");

        canv_binbybin_pT->Update();
        canv_binbybin_pT->Write();
*/
       	dir_pT->cd();
        canv_pT->Draw();
        canv_pT->cd();
        SM_pT->Draw();
        Zprime_pT->Draw("sames");
        KK_pT->Draw("sames");
        leg_pT->Draw("sames");
        canv_pT->Update();
        canv_pT->Write();

        dir_pT_relDiff->cd();
        canv_pT_relDiff->Draw();
	canv_pT_relDiff->cd();
        KK_positive_diff_pT->Draw();
        KK_negative_diff_pT->Draw("sames");
        Zprime_positive_diff_pT->Draw("sames");
        Zprime_negative_diff_pT->Draw("sames");
        zeroLine_pT->Draw("sames");
        leg_binbybin_pT->Draw("sames");
        canv_pT_relDiff->Update();
        canv_pT_relDiff->Write();
	
	cout << "\n---finalizing---\n" << endl;



	// .root
	//binbybinHistFile->ls();
	binbybinHistFile->Write();
	// binbybinHistFile->Close();

	// .eps
	canv_binbybin_mHat->Print(binbybinEpsFileName_mHat.c_str());
	canv_binbybin_pT->Print(binbybinEpsFileName_pT.c_str());

	// wait so the plots can hold on
	canv_binbybin_mHat->WaitPrimitive();
	canv_binbybin_pT->WaitPrimitive();

	// Done.
	return 0;
}

