/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/KKvsZE6vsSMstudy/";
	string outFileName = dirName + "Graviton_Kinematics";
	cout << "\n" << endl;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	int cutSelector = 1;
	cout << "select cuts to apply: no cuts (-1) all cuts (0), only mHat cut (1), only pT cut (2), only eta cut (3)..." << endl;
	cin >> cutSelector;
	switch(cutSelector) {
		case -1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 invMassCut = 0.;
			 break;
		case  0: break;
		case  1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  2: invMassCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  3: invMassCut = 0.;
			 pTransverseCut = 0.;
			 break;
		default: cout << "wrong option cutSelector = " << cutSelector << "\t set to default cutSelector=0" << endl;
			 break;
	}
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);



	// choose reference frame
	int referenceFrameSelector = 0;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	string cosThetaTitle;
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}


	int gravitonSelector = 0;
	cout << "choose to take only 1M_G(1), 1M_G+1M_SM(2) or 10k_G+1M_SM(3)" << endl;
	cin >> gravitonSelector;
	if (gravitonSelector != 1 && gravitonSelector != 2 && gravitonSelector != 3) {
		cout << "invalid option, setting to default = 1" << endl;
		gravitonSelector = 1;
	}
	string sGravitonSampleSize = "";
	if (gravitonSelector==3) {
		cout << "enter the small graviton sample size: 10k / 50k / 100k .... ";
		cin >> sGravitonSampleSize;
		cout << endl;
		if (sGravitonSampleSize!="10k" && sGravitonSampleSize!="50k" && sGravitonSampleSize!="100k") {
			cout << "invalid option, setting to default = 50k" << endl;
			sGravitonSampleSize = "50k";
		}
	}


	
	// declare kinematics histograms and root items
	string kinematicsRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".root");
	string kinematicsEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".eps");


	TFile *kinematicsHistFile = new TFile(kinematicsRootFileName.c_str(),"recreate");
	TCanvas *canv = new TCanvas("canv","Kaluza-Klein",0,0,1400,800);
	canv->Divide(1,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=canv->cd(1);
	pad1->SetLogy();	


	// boundries and bins of histograms
	Int_t invMnBins = 100;
	Int_t cosThetanBins = 110;
        Double_t invMmin = 1000.,	invMmax = 6000.;
	Double_t cosThetamin = -1.1,	cosThetamax = 1.1;

	if (cutSelector != 0) {
		invMnBins = 500;
		invMmin = 5.;
		invMmax = 6000.;

		cosThetanBins = 120;
		cosThetamin   = -1.1;
		cosThetamax   = 1.1;
	}


	
	// titles for the cos(theta) plots
	string title  = "";
	string xtitle = "cos#theta ' (" + cosThetaTitle + " frame)";


	// declaration of histograms
	//----- SM -----------------------------------------------------------------------------------------------------
	TH1 *SM_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	SM_invMass->SetLineStyle(5);
	SM_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SM_invMass->SetYTitle("Events");


	TH1 *SM_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	SM_cosTheta->SetLineStyle(5);
	SM_cosTheta->SetXTitle(xtitle.c_str());
	SM_cosTheta->SetYTitle("#frac{Events}{#sum Events}");
	

	//----- KK -----------------------------------------------------------------------------------------------------
	TH1 *KK_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	KK_invMass->SetLineStyle(1);
	KK_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK_invMass->SetYTitle("Events");


	TH1 *KK_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	KK_cosTheta->SetLineStyle(1);
	KK_cosTheta->SetXTitle(xtitle.c_str());
	KK_cosTheta->SetYTitle("#frac{Events}{#sum Events}");

	
	//----- Z' -----------------------------------------------------------------------------------------------------
	TH1 *Zprime_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	Zprime_invMass->SetLineStyle(3);
	Zprime_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	Zprime_invMass->SetYTitle("Events");


	TH1 *Zprime_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	Zprime_cosTheta->SetLineStyle(3);
	Zprime_cosTheta->SetXTitle(xtitle.c_str());
	Zprime_cosTheta->SetYTitle("#frac{Events}{#sum Events}");


	//----- GR -----------------------------------------------------------------------------------------------------
	TH1 *GR_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	GR_invMass->SetLineStyle(7);
	GR_invMass->SetLineWidth(2);
	GR_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	GR_invMass->SetYTitle("Events");
	TH1 *SMforGR_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	SMforGR_invMass->SetLineStyle(7);
	SMforGR_invMass->SetLineWidth(2);
	SMforGR_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SMforGR_invMass->SetYTitle("Events");


	TH1 *GR_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	GR_cosTheta->SetLineStyle(7);
	GR_cosTheta->SetLineWidth(2);
	GR_cosTheta->SetXTitle(xtitle.c_str());
	GR_cosTheta->SetYTitle("#frac{Events}{#sum Events}");
	TH1 *SMforGR_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	SMforGR_cosTheta->SetLineStyle(7);
	SMforGR_cosTheta->SetLineWidth(2);
	SMforGR_cosTheta->SetXTitle(xtitle.c_str());
	SMforGR_cosTheta->SetYTitle("#frac{Events}{#sum Events}");


	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg1 = new TLegend(0.7,0.5,0.96,0.7);
	leg1->AddEntry(SM_invMass,"SM","L");
	leg1->AddEntry(KK_invMass,"KK","L");
	leg1->AddEntry(Zprime_invMass,"Z'_{SM}","L");
	leg1->AddEntry(GR_invMass,"G","L");


	TLegend *leg4 = new TLegend(0.3,0.7,0.56,0.9);
	leg4->AddEntry(SM_cosTheta,"SM","L");
	leg4->AddEntry(KK_cosTheta,"KK","L");
	leg4->AddEntry(Zprime_cosTheta,"Z'_{SM}","L");
	leg4->AddEntry(GR_cosTheta,"G","L");
	//----------------------------------------------------------------------------------------------------------
	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, pTransverse, Rapidity, cosTh=0;


	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	

	string treeType = "testTree";
	int treeTypeSelector = 1;
	cout << "select tree type: testTree(1), testTree_allPairs(2)..." << endl;
	cin >> treeTypeSelector;
	switch(treeTypeSelector) {
		case 1: treeType = "testTree"; break;
		case 2: treeType = "testTree_allPairs"; break;
		default: cout << "wrong option: treeTypeSelector=" << treeTypeSelector << "\t setting it to default: treeTypeSelector=1" << endl;
			treeType = "testTree";
			break;
	}


	vector<Double_t> vec_SM_mHat;
	vector<Double_t> vec_SM_cosTheta;
	vector<Double_t> vec_KK_mHat;
	vector<Double_t> vec_KK_cosTheta;
	vector<Double_t> vec_E6_mHat;
	vector<Double_t> vec_E6_cosTheta;
	vector<Double_t> vec_GR_mHat;
	vector<Double_t> vec_GR_cosTheta;


	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=5 ; f++) {
		//-------------------------------------------------------------------------------------------------------------------
		// read the trees and put all good lepton pairs into 2 vectors
		if (f==1) { // SM
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"SM_extractedParticles_2trees_1Msample.root");
		}
		else if (f==2) { // KK
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"KK_extractedParticles_2trees_1Msample.root");
		}
		else if (f==3) { // E6
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"E6_extractedParticles_2trees_1Msample.root");
		}
		else if (f==4) { // GR
			if(gravitonSelector==2) {
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"GR_extractedParticles_2trees_1Msample.root");
			}
			else if(gravitonSelector==3) {
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"GR_extractedParticles_2trees_" + sGravitonSampleSize + "_EventsSample.root");
			}
		}
		else if (f==5) { // GR
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"SMforGR_extractedParticles_2trees_1Msample.root");
		}

		Nevents = anlz.loopOverEvents(filePath, treeType.c_str(), 0,0);
		cout << "Nevents = " << Nevents << endl;
		// get all the good pairs (after cuts)
		Nleptons = anlz.getAllLeptons(allgoodLeptons);
		Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
		//-------------------------------------------------------------------------------------------------------------------
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			pTransverse = anlz.pT(allgoodLeptons[k], allgoodaLeptons[k]);
			Rapidity = anlz.y(allgoodLeptons[k], allgoodaLeptons[k]);
			cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
			if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
			else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
			// fill the histos
			if (f==1) {
				SM_invMass->Fill( (Double_t)invariantMass );
				vec_SM_mHat.push_back(invariantMass);
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					SM_cosTheta->Fill( (Double_t)cosTh );
					vec_SM_cosTheta.push_back(cosTh);
				}	
			}

			else if (f==2) {
				KK_invMass->Fill( (Double_t)invariantMass );
				vec_KK_mHat.push_back(invariantMass);
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					KK_cosTheta->Fill( (Double_t)cosTh );
					vec_KK_cosTheta.push_back(cosTh);
				}
			}
			else if (f==3) {
				Zprime_invMass->Fill( (Double_t)invariantMass );
				vec_E6_mHat.push_back(invariantMass);
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					Zprime_cosTheta->Fill( (Double_t)cosTh );
					vec_E6_cosTheta.push_back(cosTh);
				}
			}
			else if (f==4 || f==5) {
				if(f==4) {
					GR_invMass->Fill( (Double_t)invariantMass );
					vec_GR_mHat.push_back(invariantMass);
				}
				else {
					if (gravitonSelector==2 || gravitonSelector==3) {
						SMforGR_invMass->Fill( (Double_t)invariantMass );
						vec_GR_mHat.push_back(invariantMass);
					}
				}
				if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
					if(f==4) {
						SMforGR_cosTheta->Fill( (Double_t)cosTh );
						vec_GR_cosTheta.push_back(cosTh);
					}
					else {
						if (gravitonSelector==2 || gravitonSelector==3) {
							SMforGR_cosTheta->Fill( (Double_t)cosTh );
							vec_GR_cosTheta.push_back(cosTh);
						}
					}
				}
			}
		}
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}

	Double_t *array_SM_mHat = (Double_t *)malloc( (Int_t)vec_SM_mHat.size() * sizeof(Double_t) );
	Double_t *array_SM_cosTheta = (Double_t *)malloc( (Int_t)vec_SM_cosTheta.size() * sizeof(Double_t) );
	// sort the vectors:
	sort(vec_SM_mHat.begin(), vec_SM_mHat.end());
	sort(vec_SM_cosTheta.begin(), vec_SM_cosTheta.end());
	// fill the sorted arrays:
	for (int j=0 ; j<(int)vec_SM_mHat.size()     ; ++j) { array_SM_mHat[j]     = vec_SM_mHat[j]; }
	for (int j=0 ; j<(int)vec_SM_cosTheta.size() ; ++j) { array_SM_cosTheta[j] = vec_SM_cosTheta[j]; }


	Double_t *array_KK_mHat = (Double_t *)malloc( (Int_t)vec_KK_mHat.size() * sizeof(Double_t) );
	Double_t *array_KK_cosTheta = (Double_t *)malloc( (Int_t)vec_KK_cosTheta.size() * sizeof(Double_t) );
	// sort the vectors:
	sort(vec_KK_mHat.begin(), vec_KK_mHat.end());
	sort(vec_KK_cosTheta.begin(), vec_KK_cosTheta.end());
	// fill the sorted arrays:
	for (int j=0 ; j<(int)vec_KK_mHat.size()     ; ++j) { array_KK_mHat[j]     = vec_KK_mHat[j]; }
	for (int j=0 ; j<(int)vec_KK_cosTheta.size() ; ++j) { array_KK_cosTheta[j] = vec_KK_cosTheta[j]; }


	Double_t *array_E6_mHat = (Double_t *)malloc( (Int_t)vec_E6_mHat.size() * sizeof(Double_t) );
	Double_t *array_E6_cosTheta = (Double_t *)malloc( (Int_t)vec_E6_cosTheta.size() * sizeof(Double_t) );
	// sort the vectors:
	sort(vec_E6_mHat.begin(), vec_E6_mHat.end());
	sort(vec_E6_cosTheta.begin(), vec_E6_cosTheta.end());
	// fill the sorted arrays:
	for (int j=0 ; j<(int)vec_E6_mHat.size()     ; ++j) { array_E6_mHat[j]     = vec_E6_mHat[j]; }
	for (int j=0 ; j<(int)vec_E6_cosTheta.size() ; ++j) { array_E6_cosTheta[j] = vec_E6_cosTheta[j]; }


	Double_t *array_GR_mHat = (Double_t *)malloc( (Int_t)vec_GR_mHat.size() * sizeof(Double_t) );
	Double_t *array_GR_cosTheta = (Double_t *)malloc( (Int_t)vec_GR_cosTheta.size() * sizeof(Double_t) );
	// sort the vectors:
	sort(vec_GR_mHat.begin(), vec_GR_mHat.end());
	sort(vec_GR_cosTheta.begin(), vec_GR_cosTheta.end());
	// fill the sorted arrays:
	for (int j=0 ; j<(int)vec_GR_mHat.size()     ; ++j) { array_GR_mHat[j]     = vec_GR_mHat[j]; }
	for (int j=0 ; j<(int)vec_GR_cosTheta.size() ; ++j) { array_GR_cosTheta[j] = vec_GR_cosTheta[j]; }


	// do the unbinned ks test:
	Double_t unbinnedKStest_SM_mHat = TMath::KolmogorovTest((Int_t)vec_GR_mHat.size(), array_GR_mHat, (Int_t)vec_SM_mHat.size(), array_SM_mHat ,"");
	Double_t unbinnedKStest_E6_mHat = TMath::KolmogorovTest((Int_t)vec_GR_mHat.size(), array_GR_mHat, (Int_t)vec_E6_mHat.size(), array_E6_mHat ,"");
	Double_t unbinnedKStest_KK_mHat = TMath::KolmogorovTest((Int_t)vec_GR_mHat.size(), array_GR_mHat, (Int_t)vec_KK_mHat.size(), array_KK_mHat ,"");

	Double_t unbinnedKStest_SM_cosTheta = TMath::KolmogorovTest((Int_t)vec_GR_cosTheta.size(), array_GR_cosTheta, (Int_t)vec_SM_cosTheta.size(), array_SM_cosTheta ,"");
	Double_t unbinnedKStest_E6_cosTheta = TMath::KolmogorovTest((Int_t)vec_GR_cosTheta.size(), array_GR_cosTheta, (Int_t)vec_E6_cosTheta.size(), array_E6_cosTheta ,"");
	Double_t unbinnedKStest_KK_cosTheta = TMath::KolmogorovTest((Int_t)vec_GR_cosTheta.size(), array_GR_cosTheta, (Int_t)vec_KK_cosTheta.size(), array_KK_cosTheta ,"");
	
	cout << endl;
	cout << "--------------------------------------------------------"<< endl;
	cout << "mHat     :   SM unbinned KStest = " << unbinnedKStest_SM_mHat << endl;
	cout << "mHat     :   E6 unbinned KStest = " << unbinnedKStest_E6_mHat << endl;
	cout << "mHat     :   KK unbinned KStest = " << unbinnedKStest_KK_mHat << endl;

	cout << "cosTheta :   SM unbinned KStest = " << unbinnedKStest_SM_cosTheta << endl;
	cout << "cosTheta :   E6 unbinned KStest = " << unbinnedKStest_E6_cosTheta << endl;
	cout << "cosTheta :   KK unbinned KStest = " << unbinnedKStest_KK_cosTheta << endl;
	cout << "--------------------------------------------------------" << endl;
	cout << endl;




	// add the SMforGR and the GR histograms
	if (gravitonSelector==2 || gravitonSelector==3) {
		//GR_cosTheta->Scale("sigma_GR / sigma_SM"); // ????????????????????????????????????????????????????????
		GR_cosTheta->Add(SMforGR_cosTheta);
		//GR_invMass->Scale("sigma_GR / sigma_SM"); // ?????????????????????????????????????????????????????????
	        GR_invMass->Add(SMforGR_invMass);
	}



	// normalize the cos(theta) distributions
	Double_t normalization = 1.;
	normalization = SM_cosTheta->Integral();
	if (normalization != 0) { SM_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for SM_cosTheta == 0" << endl; }
	
	normalization = Zprime_cosTheta->Integral();
	if (normalization != 0) { Zprime_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for Zprime_cosTheta == 0" << endl; }
	
	normalization = KK_cosTheta->Integral();
	if (normalization != 0) { KK_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for KK_cosTheta == 0" << endl; }

	normalization = GR_cosTheta->Integral();
	if (normalization != 0) { GR_cosTheta->Scale(1./normalization); }
	else { cerr << "normalization for GR_cosTheta == 0" << endl; }

	cout << "\n" << endl;



	//--------------------------------------------------------------------------------------


	// scale reference histos
	// for 1M KK events, if the SM sample size is also 1M then the histos should be scaled by 1.2
	// for 1M KK events, if the E6 sample size is also 1M then the histos should be scaled by 1.15
	SM_invMass->Scale(1.2);
	Zprime_invMass->Scale(1.15);
	if (gravitonSelector==2) {
		GR_invMass->Scale(1.2); // ?????????????????????????????????????????????????????????????????????????????????????
		// should be less than 1.2 ?????????????????????????????????????????????????????????????????????????????????????
	}



	// find the maximum:
	Double_t maxHist = KK_invMass->GetMaximum();
	if (Zprime_invMass->GetMaximum() > maxHist) { maxHist = Zprime_invMass->GetMaximum(); }
	if (GR_invMass->GetMaximum() > maxHist) { maxHist = GR_invMass->GetMaximum(); }

	// draw the histos
	canv->cd(1);
	SM_invMass->SetMaximum(2*KK_invMass->GetMaximum());
	SM_invMass->Draw();
	Zprime_invMass->Draw("sames");
	KK_invMass->Draw("sames");
	GR_invMass->Draw("sames");
	leg1->Draw("sames");

	canv->cd(2);
	SM_cosTheta->SetMaximum(1.1*SM_cosTheta->GetMaximum());
	SM_cosTheta->Draw();
	Zprime_cosTheta->Draw("sames");
	KK_cosTheta->Draw("sames");
	GR_cosTheta->Draw("sames");
	leg4->Draw("sames");
	
	canv->Update();

	cout << "\n---finalizing---\n" << endl;
	
	// finalize the graphics
	canv->Print(kinematicsRootFileName.c_str(),"root");
	canv->Update();
	canv->Print(kinematicsEpsFileName.c_str());
	kinematicsHistFile->Write();
	

	// wait so the plots can hold on
	canv->WaitPrimitive();


	// Done.
	return 0;
}

