/************************************************************/
/* an example program to demonstrate a basic use of pythia8 */
/* for an LHC setup                                         */

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------


using namespace Moses1;
using namespace Pythia8;

int main() {
	// Generator. Process selection. LHC initialization. Histogram.
	Pythia pythia;

	// Allow no substructure in e+- beams: normal for corrected LEP data.
	pythia.readString("PDF:lepton = off");

	// to switch on the process ffbar--->gamma/Z
	pythia.readString("WeakSingleBoson:ffbar2gmZ = on");
	pythia.readString("WeakZ0:gmZmode = 1"); // only photon propagator (can be usefull for tests)
	//pythia.readString("WeakZ0:gmZmode = 2"); // only Z0 propagator (can be usefull for tests)

//	pythia.readString("WeakSingleBoson:ffbar2ffbar(s:gm) = on");

	// rule out all gamma/Z decays except for muons
	pythia.readString("22:onMode = off"); // switch off all of the gamma decay modes
	pythia.readString("22:onIfAny = 13"); // switch on the gamma->mu-mu+ decay mode only
	pythia.readString("23:onMode = off"); // switch off all of the Z0 decay modes
	pythia.readString("23:onIfAny = 13"); // switch on the Z0->mu-mu+ decay mode only

	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
	}

	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 1");
	pythia.readString("PhaseSpace:mHatMax = 20");


	//with time-based seed.
	pythia.readString("Random:setSeed = on");
	pythia.readString("Random:seed = 0");

	// initialization for low energies below the Z0 threshold
	pythia.init(11, -11, 20.);
	cout << "Initialization for LEP" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();

	// file
	commonOperations commonOps;
	ofstream muFile;
	string fileName = commonOps.checkANDsetFilepath("MOSESDATA", "/pythia_muons.PYT"); 
	muFile.open(fileName.c_str());

	// run
	int nEvents = 200;

	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;
	cout << "\n\nenter the number of MUON-PAIRS to loop over (integer>0, default is 200): ";
	cin >> nEvents;
	cout << "start running and booking events to files..." << endl;

	int mod = (int)nEvents/10;
	int evt=1;
	while (evt <= nEvents) {
		pythia.next();
		muFile << evt << "\t" << pythia.info.mHat() << "\n";
		// this is a 2->1->2 process therefore it is not possible to get pythia.info.thetaHat() unlike pythia.info.mHat()
		// for more details, see:           StandardModel_qqbar2gmZ2mumubar/pythiaInternalMain.cxx
		if (evt%mod==0) {
			cout << "event: " << evt << endl;
			//pythia.process.list();
		}
		//if (evt%mod==0) {pythia.process.list();}
		evt++;
	}
	cout << "1 file was written" << endl;
	muFile.close();
	pythia.statistics();
}

