/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
#include "pythiaPlugin.h"

using namespace std;
using namespace Pythia8; 

namespace Moses1 {



void eebar2mumubar::initProc() {
	cout << "initProc ok" << endl;
}


// Evaluate sigmaKin(sHat); first step when inflavours unknown. 
void eebar2mumubar::sigmaKin() {	

	m_thetaDistribution = 0.;
	// Check that above threshold.
	m_isPhysical = true;
	if (mH < m3 + m4 + MASSMARGIN) {
		m_isPhysical = false;
		return;
	}
	// calculate sigma
	m_cosThe = (tH-uH)/sH;
	m_thetaDistribution = 1. + pow2(m_cosThe);
}



// Evaluate sigmaHat(sHat); first step when inflavours unknown. 
double eebar2mumubar::sigmaHat() {
	// Fail if below threshold.
	if (!m_isPhysical) {cout << "not physical" << endl; return 0.;}

	// Cross section as already calculated.
	m_sigma = (2./sH) * (2.*M_PI) * pow2(alpEM)/(4.*sH) * m_thetaDistribution;
	return m_sigma;
}



// Select identity, colour and anticolour.
void eebar2mumubar::setIdColAcol() {

	// Set outgoing flavours.
	id3 = (id1 > 0) ? m_idNew : -m_idNew;
	setId(id1, id2, id3, -id3);

	// Colour flow topologies. Swap when antiquarks.
	setColAcol(0,0, 0,0, 0,0, 0,0);
}



}
