#ifndef PYTHIASEMIEXTERNALPLUGIN_h
#define PYTHIASEMIEXTERNALPLUGIN_h
/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Pythia8; 

namespace Moses1 {


// A derived class for f fbar -> gamma*/Z0 -> F Fbar, for one heavy F.
class PYTHIA8gmZ2mumubar : public Sigma2Process {
	public:

		// Constructor.
		PYTHIA8gmZ2mumubar(int idIn, int codeIn) : idNew(idIn), codeSave(codeIn) {}

		// initialize the process:
		void initProc();

		// Calculate flavour-independent parts of cross section.
		virtual void sigmaKin();

		// Evaluate sigmaHat(sHat). 
		virtual double sigmaHat();

		// Select flavour, colour and anticolour.
		virtual void setIdColAcol();

		// Evaluate weight for W decay angles in top decay (else inactive).
		virtual double weightDecay( Event& process, int iResBeg, int iResEnd); 

		// Info on the subprocess.
		virtual string name()       const {return nameSave;}
		virtual int    code()       const {return codeSave;}
		virtual string inFlux()     const {return "ffbarSame";}
		virtual bool   isSChannel() const {return true;}
		virtual int    id3Mass()    const {return idNew;}
		virtual int    id4Mass()    const {return idNew;}
		virtual int    resonanceA() const {return 23;}


	private:
		
		double sigma;
		// Values stored for process type. Z parameters for propagator.
		int    idNew, codeSave, gmZmode;
		string nameSave;
		bool   isPhysical;
		double ef, vf, af, mRes, GammaRes, m2Res, GamMRat, thetaWRat,
				mr, betaf, cosThe, gamProp, intProp, resProp, openFracPair;
};




}
#endif
