/**************************************************/
/* a class for general analysis of pythia's event */
/**************************************************/
#include "analyseEvent.h"

using namespace std;
using namespace Pythia8;

namespace Moses1 {

extractFromEventRecord::extractFromEventRecord(){
	m_eventNumber = 1;
	m_doSmearing = false;
}

extractFromEventRecord::~extractFromEventRecord(){}



void extractFromEventRecord::setSmear(double average, double sigma, double resolution, double pTTrackScale, bool printChanges){
	m_smearing.settings(average, sigma, resolution, pTTrackScale);
        m_doSmearing = true;
        m_printChanges = printChanges;

	info_trueEventsWithOnePair = 0;
	info_smearedEventsWithOnePair = 0;
	info_trueEventsPassingCuts = 0;
	info_smearedEventsPassingCuts = 0;
	info_eventsWithFlippedCharge = 0;
	info_eventsWithMoreThanOnePair = 0;
	info_trueEventsWithOnePairFailingCuts = 0;
	info_smearedEventsWithOnePairFailingCuts = 0;
}

void extractFromEventRecord::printInfo(){
	cout << "trueEventsWithOnePair     = " << info_trueEventsWithOnePair << endl;
	cout << "smearedEventsWithOnePair  = " << info_smearedEventsWithOnePair << endl;
	cout << "trueEventsPassingCuts     = " << info_trueEventsPassingCuts << endl;
	cout << "smearedEventsPassingCuts  = " << info_smearedEventsPassingCuts << endl;
	cout << "eventsWithFlippedCharge   = " << info_eventsWithFlippedCharge << endl;
	cout << "eventsWithMoreThanOnePair = " << info_eventsWithMoreThanOnePair << endl;
	cout << "trueEventsWithOnePairFailingCuts    = " << info_trueEventsWithOnePairFailingCuts << endl;
	cout << "smearedEventsWithOnePairFailingCuts = " << info_smearedEventsWithOnePairFailingCuts << endl;
}


void extractFromEventRecord::setOutputFile(string envPath, string fileName, bool fullRecord) { // flat file
	m_filePath = m_commonOps.checkANDsetFilepath(envPath, fileName);
	m_outFile.open(m_filePath.c_str());
	m_fullRecord = fullRecord;
}

void extractFromEventRecord::setOutputFile(string envPath, string fileName, bool fullRecord, string treeName) { // overload - tree file

	m_fullRecord = fullRecord;

	m_filePath = m_commonOps.checkANDsetFilepath(envPath, fileName);
	m_treeFile = new TFile(m_filePath.c_str(),"recreate");

	if (m_fullRecord) {
		m_fullRecordTree = new TTree(treeName.c_str(),treeName.c_str());
		
		m_fullRecordTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/I");
		m_fullRecordTree->Branch("id",&m_id,"m_id/I");
		m_fullRecordTree->Branch("m",&m_m,"m_m/D");
		m_fullRecordTree->Branch("e",&m_e,"m_e/D");
		m_fullRecordTree->Branch("px",&m_px,"m_px/D");
		m_fullRecordTree->Branch("py",&m_py,"m_py/D");
		m_fullRecordTree->Branch("pz",&m_pz,"m_pz/D");
		m_fullRecordTree->Branch("pAbs",&m_pAbs,"m_pAbs/D");
		m_fullRecordTree->Branch("pT",&m_pT,"m_pT/D");
		m_fullRecordTree->Branch("pPlus",&m_pPlus,"m_pPlus/D");
		m_fullRecordTree->Branch("pMinus",&m_pMinus,"m_pMinus/D");
		m_fullRecordTree->Branch("theta",&m_theta,"m_theta/D");
		m_fullRecordTree->Branch("phi",&m_phi,"m_phi/D");
		m_fullRecordTree->Branch("y",&m_y,"m_y/D");
		m_fullRecordTree->Branch("eta",&m_eta,"m_eta/D");
		//--------------- scaling variables -----------------
		m_fullRecordTree->Branch("xq",&m_xq,"m_xq/D");
		m_fullRecordTree->Branch("xqbar",&m_xqbar,"m_xqbar/D");
		//---------------------------------------------------


		//--------- a tree for all pairs (also for the ones that didnt pass the cuts)
		treeName += "_allPairs";
                m_fullRecordAllPairsTree = new TTree(treeName.c_str(),treeName.c_str());

                m_fullRecordAllPairsTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/I");
                m_fullRecordAllPairsTree->Branch("id",&m_id,"m_id/I");
                m_fullRecordAllPairsTree->Branch("m",&m_m,"m_m/D");
                m_fullRecordAllPairsTree->Branch("e",&m_e,"m_e/D");
                m_fullRecordAllPairsTree->Branch("px",&m_px,"m_px/D");
                m_fullRecordAllPairsTree->Branch("py",&m_py,"m_py/D");
                m_fullRecordAllPairsTree->Branch("pz",&m_pz,"m_pz/D");
                m_fullRecordAllPairsTree->Branch("pAbs",&m_pAbs,"m_pAbs/D");
                m_fullRecordAllPairsTree->Branch("pT",&m_pT,"m_pT/D");
                m_fullRecordAllPairsTree->Branch("pPlus",&m_pPlus,"m_pPlus/D");
                m_fullRecordAllPairsTree->Branch("pMinus",&m_pMinus,"m_pMinus/D");
                m_fullRecordAllPairsTree->Branch("theta",&m_theta,"m_theta/D");
                m_fullRecordAllPairsTree->Branch("phi",&m_phi,"m_phi/D");
                m_fullRecordAllPairsTree->Branch("y",&m_y,"m_y/D");
                m_fullRecordAllPairsTree->Branch("eta",&m_eta,"m_eta/D");
                //--------------- scaling variables -----------------
                m_fullRecordAllPairsTree->Branch("xq",&m_xq,"m_xq/D");
                m_fullRecordAllPairsTree->Branch("xqbar",&m_xqbar,"m_xqbar/D");
                //---------------------------------------------------


		//--------- a tree for all true pairs (also for the ones that didnt pass the cuts)
                treeName += "_True";
                m_fullRecordAllTruePairsTree = new TTree(treeName.c_str(),treeName.c_str());

                m_fullRecordAllTruePairsTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/I");
                m_fullRecordAllTruePairsTree->Branch("id",&m_tru_id,"m_tru_id/I");
                m_fullRecordAllTruePairsTree->Branch("m",&m_tru_m,"m_tru_m/D");
                m_fullRecordAllTruePairsTree->Branch("e",&m_tru_e,"m_tru_e/D");
                m_fullRecordAllTruePairsTree->Branch("px",&m_tru_px,"m_tru_px/D");
                m_fullRecordAllTruePairsTree->Branch("py",&m_tru_py,"m_tru_py/D");
                m_fullRecordAllTruePairsTree->Branch("pz",&m_tru_pz,"m_tru_pz/D");
                m_fullRecordAllTruePairsTree->Branch("pAbs",&m_tru_pAbs,"m_tru_pAbs/D");
                m_fullRecordAllTruePairsTree->Branch("pT",&m_tru_pT,"m_tru_pT/D");
                m_fullRecordAllTruePairsTree->Branch("pPlus",&m_tru_pPlus,"m_tru_pPlus/D");
                m_fullRecordAllTruePairsTree->Branch("pMinus",&m_tru_pMinus,"m_tru_pMinus/D");
                m_fullRecordAllTruePairsTree->Branch("theta",&m_tru_theta,"m_tru_theta/D");
                m_fullRecordAllTruePairsTree->Branch("phi",&m_tru_phi,"m_tru_phi/D");
                m_fullRecordAllTruePairsTree->Branch("y",&m_tru_y,"m_tru_y/D");
                m_fullRecordAllTruePairsTree->Branch("eta",&m_tru_eta,"m_tru_eta/D");
	}
	else {
		m_reducedRecordTree = new TTree(treeName.c_str(),treeName.c_str());

		m_reducedRecordTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/D");
		m_reducedRecordTree->Branch("id",&m_id,"m_id/D");
		m_reducedRecordTree->Branch("m",&m_m,"m_m/D");


		//--------- a tree for all pairs (also for the ones that didnt pass the cuts)
		treeName += "_allPairs";
                m_reducedRecordAllPairsTree = new TTree(treeName.c_str(),treeName.c_str());

                m_reducedRecordAllPairsTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/D");
                m_reducedRecordAllPairsTree->Branch("id",&m_id,"m_id/D");
                m_reducedRecordAllPairsTree->Branch("m",&m_m,"m_m/D");

		
		//--------- a tree for all pairs (also for the ones that didnt pass the cuts)
                treeName += "_allPairs";
                m_reducedRecordAllPairsTree = new TTree(treeName.c_str(),treeName.c_str());

                m_reducedRecordAllTruePairsTree->Branch("eventNumber",&m_eventNumber,"m_eventNumber/D");
                m_reducedRecordAllTruePairsTree->Branch("id",&m_id,"m_tru_id/D");
                m_reducedRecordAllTruePairsTree->Branch("m",&m_m,"m_tru_m/D");
	}
}




void extractFromEventRecord::extract2vector(Particle &particle) {
	m_particleVec.push_back(particle);
	if (particle.id() > 0) m_Nparticles++;
	else if (particle.id() < 0) m_Nantiparticles++;
}

void extractFromEventRecord::extract2vectors(Particle &particle) {
	if (particle.id() > 0) { m_Vec.push_back(particle); }
	else { m_antiVec.push_back(particle); }	
}

void extractFromEventRecord::extract2vectors(Particle &particle, bool doSmear) {

        if(doSmear)
        {
                double pTSmeared = m_smearing.pTsmear( particle.pT() );
                double pxSmeared = m_smearing.pT2px(pTSmeared, particle.phi());
                double pySmeared = m_smearing.pT2py(pTSmeared, particle.phi());
                double pzSmeared = m_smearing.pT2pz(pTSmeared, particle.theta());
                double eSmeared  = m_smearing.pT2e(pTSmeared, particle.theta());

                // pring the changes
                if(m_printChanges)
                {
                        m_smearing.compare2true( particle.px(), particle.py(), particle.pz(), particle.e() );
                        if(pTSmeared<0) {cout << "smeared pT=" << pTSmeared  << " ==> charge flip !" << endl;}
                }

                // update the smeared four momentum in the particle
                particle.px(pxSmeared);
                particle.py(pySmeared);
                particle.pz(pzSmeared);
                particle.e(eSmeared);
                // flip the charge and the id due to resolution
                int idOrig = particle.id();
                if(pTSmeared<0)
                {
                        particle.id(-1*idOrig);
                }

        }

        if (particle.id() > 0) { m_Vec.push_back(particle); }
        else { m_antiVec.push_back(particle); }
}


void extractFromEventRecord::registerParticle(Particle &particle) { // register particles to flat file
	m_outFile << particle.id();
	m_outFile << " " << particle.m();
	if (m_fullRecord) {
		m_outFile << " " << particle.e() << " " << particle.px() << " " << particle.py() << " " << particle.pz();
		m_outFile << " " << particle.pAbs() << " " << particle.pT() << " " << particle.pPos() << " " << particle.pNeg();
		m_outFile << " " << particle.theta() << " " << particle.phi();
		m_outFile << " " << particle.y() << " " << particle.eta();
	}
}

void extractFromEventRecord::registerParticle(Particle &particle, bool write2tree) { // overload - register particles to tree file
	m_particle = particle;

	m_id = m_particle.id();
	m_m = m_particle.m();
	m_e = m_particle.e();
	m_px = m_particle.px();
	m_py = m_particle.py();
	m_pz = m_particle.pz();
	m_pAbs = m_particle.pAbs();
	m_pT = m_particle.pT();
	m_pPlus = m_particle.pPos();
	m_pMinus = m_particle.pNeg();
	m_theta = m_particle.theta();
	m_phi = m_particle.phi();
	m_y = m_particle.y();
	m_eta = m_particle.eta();

	if (m_fullRecord) {m_fullRecordTree->Fill();}
	else {m_reducedRecordTree->Fill();}
}

void extractFromEventRecord::registerParticle(Particle &particle, bool writeXbjorken, bool write2tree) { // overload - register particles to tree file
	m_particle = particle;

	m_id = m_particle.id();
	m_m = m_particle.m();
	m_e = m_particle.e();
	m_px = m_particle.px();
	m_py = m_particle.py();
	m_pz = m_particle.pz();
	m_pAbs = m_particle.pAbs();
	m_pT = m_particle.pT();
	m_pPlus = m_particle.pPos();
	m_pMinus = m_particle.pNeg();
	m_theta = m_particle.theta();
	m_phi = m_particle.phi();
	m_y = m_particle.y();
	m_eta = m_particle.eta();
	
	//--------------- scaling variables -----------------
	m_xq = m_xBjorken_q;
	m_xqbar = m_xBjorken_qbar;
	//---------------------------------------------------

	if (m_fullRecord) {m_fullRecordTree->Fill();}
	else {m_reducedRecordTree->Fill();}
}


void extractFromEventRecord::registerParticle(Particle &particle, bool writeXbjorken, bool write2tree, bool passCuts) { // overload

	// if passCuts==true then the "true" pair passed the cuts
	// if passCuts==false then the "true" pair did not pass the cuts
	// the "Particle &particle" is smeared

        m_particle = particle;

        m_id = m_particle.id();
        m_m = m_particle.m();
        m_e = m_particle.e();
        m_px = m_particle.px();
        m_py = m_particle.py();
        m_pz = m_particle.pz();
        m_pAbs = m_particle.pAbs();
        m_pT = m_particle.pT();
        m_pPlus = m_particle.pPos();
        m_pMinus = m_particle.pNeg();
        m_theta = m_particle.theta();
        m_phi = m_particle.phi();
        m_y = m_particle.y();
        m_eta = m_particle.eta();

        //--------------- scaling variables -----------------
        m_xq = m_xBjorken_q;
        m_xqbar = m_xBjorken_qbar;
        //---------------------------------------------------
	if (passCuts) {
		if (m_fullRecord) {
			m_fullRecordTree->Fill();
			m_fullRecordAllPairsTree->Fill();
		}
		else {
			m_reducedRecordTree->Fill();
			m_reducedRecordAllPairsTree->Fill();
		}
	}
	else {
		if (m_fullRecord) {m_fullRecordAllPairsTree->Fill();}
		else {m_reducedRecordAllPairsTree->Fill();}
	}
}



void extractFromEventRecord::registerParticle(Particle &particle, Particle &trueParticle, bool writeXbjorken, bool write2tree, bool passCuts) { // overload

        // if passCuts==true then the "true" pair passed the cuts
        // the "Particle &particle" is smeared

        m_particle = particle;
	m_trueParticle = trueParticle;

        m_id = m_particle.id();
        m_m = m_particle.m();
        m_e = m_particle.e();
        m_px = m_particle.px();
        m_py = m_particle.py();
        m_pz = m_particle.pz();
        m_pAbs = m_particle.pAbs();
        m_pT = m_particle.pT();
        m_pPlus = m_particle.pPos();
        m_pMinus = m_particle.pNeg();
        m_theta = m_particle.theta();
        m_phi = m_particle.phi();
        m_y = m_particle.y();
        m_eta = m_particle.eta();

	m_tru_id = m_trueParticle.id();
        m_tru_m = m_trueParticle.m();
        m_tru_e = m_trueParticle.e();
        m_tru_px = m_trueParticle.px();
        m_tru_py = m_trueParticle.py();
        m_tru_pz = m_trueParticle.pz();
        m_tru_pAbs = m_trueParticle.pAbs();
        m_tru_pT = m_trueParticle.pT();
        m_tru_pPlus = m_trueParticle.pPos();
        m_tru_pMinus = m_trueParticle.pNeg();
        m_tru_theta = m_trueParticle.theta();
        m_tru_phi = m_trueParticle.phi();
        m_tru_y = m_trueParticle.y();
        m_tru_eta = m_trueParticle.eta();
	

        //--------------- scaling variables -----------------
        m_xq = m_xBjorken_q;
        m_xqbar = m_xBjorken_qbar;
        //---------------------------------------------------
        if (passCuts) {
                if (m_fullRecord) {
			m_fullRecordTree->Fill();
                        m_fullRecordAllPairsTree->Fill();
			m_fullRecordAllTruePairsTree->Fill();
                }
                else {
                        m_reducedRecordTree->Fill();
                        m_reducedRecordAllPairsTree->Fill();
			m_reducedRecordAllTruePairsTree->Fill();
                }
        }
        else {
                if (m_fullRecord) {
			m_fullRecordAllPairsTree->Fill();
			m_fullRecordAllTruePairsTree->Fill();
		}
                else {
			m_reducedRecordAllPairsTree->Fill();
			m_reducedRecordAllTruePairsTree->Fill();
		}
        }
}


void extractFromEventRecord::calculateXbjorken(Event &event) {
	int mother3, mother4, tmpIndex3;
	// get the mother (proton) index
	tmpIndex3=3;
	
	while (tmpIndex3 != 1 && tmpIndex3 != 2) {
		if (event[tmpIndex3].mother1() != 0) { tmpIndex3 = event[tmpIndex3].mother1(); }
		else if (event[tmpIndex3].mother2() != 0) { tmpIndex3 = event[tmpIndex3].mother2(); }
	}
	mother3 = tmpIndex3;
	mother4 = tmpIndex3+1; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 01102009

	if (tmpIndex3 == 1) {mother4 = 2;}
	else if (tmpIndex3 == 2) {mother4 = 1;}

	if (event[3].id() > 0) { // event[3] is the quark and event[4] is the anti-quark
		m_xBjorken_q    = event[3].pAbs()/event[mother3].pAbs(); // quark
		m_xBjorken_qbar = event[4].pAbs()/event[mother4].pAbs(); // anti-quark
	}
	else if (event[3].id() < 0) { // event[4] is the quark and event[3] is the anti-quark
		m_xBjorken_q    = event[4].pAbs()/event[mother4].pAbs(); // quark
		m_xBjorken_qbar = event[3].pAbs()/event[mother3].pAbs(); // anti-quark
	}
}


void extractFromEventRecord::setCuts(runParameters &runPrm) {
	m_invMassCut = runPrm.invMassCut;
	m_pTransverseCut = runPrm.pTCut;
	m_EtaCut = runPrm.etaCut;
	m_pTsystemCut = runPrm.pTsystemCut;
	m_ySystemCut = runPrm.ySystemCut;
}



bool extractFromEventRecord::registerEvent(Event &event, int absId) {
	// clear all properties before filling again
//	bool write2tree = true;
	bool passCuts = false;
	m_particleVec.clear();
	m_antiVec.clear();
	m_Vec.clear();
	m_Nparticles = 0;
	m_Nantiparticles = 0;
	// run over all particles in the event record and fill the vector
	for (int i=0 ; i<(int)event.size() ; i++) {
		if (event[i].isFinal() && abs(event[i].id())==absId) { extract2vector(event[i]); }
	}

	//------------------------//
        // check if pass the cuts //
        //------------------------//
	// build Vec and antiVec from particleVec:
	for (int n=0 ; n<(int)m_particleVec.size() ; n++) {
		if (m_particleVec[n].id() > 0) { m_Vec.push_back(m_particleVec[n]); }
		else {m_antiVec.push_back(m_particleVec[n]); }
	}
	// check what combinations pass the cuts:
	//should be set outside the class !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_kin.setCuts(m_invMassCut, m_pTransverseCut, m_EtaCut, m_pTsystemCut, m_ySystemCut);
	for (int n=0 ; n<(int)m_Vec.size() ; n++) {
		for (int m=0 ; m<(int)m_antiVec.size() ; m++) {
			// check if this pair passes all cuts-----------------
			passCuts = m_kin.applyAllCuts(m_Vec[n], m_antiVec[m]);
			//----------------------------------------------------
			if (passCuts) { break; } // there's at least one good pair
		}
		if (passCuts) { break; } // there's at least one good pair
	}
	
	if (passCuts) {
		// register to file the number of particles and antiparticles
		m_outFile << m_Nparticles << " " << m_Nantiparticles << "\n";
		
		// loop over all the particles in the vector an register them to the file
		for (int i=0 ; i<(m_Nparticles+m_Nantiparticles) ; i++) {
			registerParticle(m_particleVec[i]);
			m_outFile << "\n";
		}
		m_outFile << "\n";
		return true;
	}
	else { return false; }
}

bool extractFromEventRecord::registerEvent(Event &event, int absId, bool write2tree) {
	// clear all properties before filling again
	bool writeXbjorken = true;
	bool passCuts = false;
	m_antiVec.clear();
	m_Vec.clear();
	// run over all particles in the event record and fill the vector
	for (int i=0 ; i<(int)event.size() ; i++) {
		if (event[i].isFinal() && abs(event[i].id())==absId) { extract2vectors(event[i]); } // build the m_Vec m_antiVec vectors
	}
	
	//------------------------//
	// check if pass the cuts //
	//------------------------//
	// check what combinations pass the cuts:
	m_kin.setCuts(m_invMassCut, m_pTransverseCut, m_EtaCut, m_pTsystemCut, m_ySystemCut);
	for (int n=0 ; n<(int)m_Vec.size() ; n++) {
		for (int m=0 ; m<(int)m_antiVec.size() ; m++) {
			// check if this pair passes all cuts-----------------
			passCuts = m_kin.applyAllCuts(m_Vec[n], m_antiVec[m]);
			//----------------------------------------------------
			if (passCuts) {
				calculateXbjorken(event);
				registerParticle(m_Vec[n], writeXbjorken, write2tree);
				registerParticle(m_antiVec[m], writeXbjorken, write2tree);
				m_eventNumber ++;
				break;
			} // there's at least one good pair
		}
		if (passCuts) { break; } // there's at least one good pair
	}
	
	if (passCuts) { return true; }
	else { return false; }
}


bool extractFromEventRecord::registerEvent(Event &event, int absId, bool write2tree, bool writeWithAllPairs) {
        // clear all properties before filling again
	bool doSmear = m_doSmearing;
	bool writeXbjorken = true;

	// get the true - count the true pairs that pass the cuts
        bool isTruePassCuts = false;
	bool isSmearedPassCuts = false;
	bool isOnePair = false;

        m_antiVec.clear();
        m_Vec.clear();

	m_VecTmp.clear();
	m_antiVecTmp.clear();
        
	// run over all particles in the event record and fill the true vectors
        for (int i=0 ; i<(int)event.size() ; i++) {
                if (event[i].isFinal() && abs(event[i].id())==absId) { extract2vectors(event[i]); } // build the TRUE m_Vec m_antiVec vectors
        }
	// store the vec and antivec (the true muons)
	m_VecTmp     = m_Vec;
	m_antiVecTmp = m_antiVec;

/*	
	double pTsystemNoCut = 14e3;
	double ySystemNoCut  = -100.;
	double mHatNoCut     = 0.;
	double pTNoCut       = 0.;
	double etaNoCut      = 1.e5;
	m_kin.setCuts(mHatNoCut, pTNoCut, etaNoCut, pTsystemNoCut, ySystemNoCut); // Set NO cuts
*/
	
	m_kin.setCuts(m_invMassCut, m_pTransverseCut, m_EtaCut, m_pTsystemCut, m_ySystemCut); // Set the cuts
	if ((int)m_Vec.size()+(int)m_antiVec.size() == 2) { // see if there is only one m+mu- "true" pair (if >1, reject the event)
		isOnePair = true;
		isTruePassCuts = true;
		isTruePassCuts = m_kin.applyAllCuts(m_Vec[0], m_antiVec[0]); // see if the mu+mu- "true" pair pass all the cuts
		info_trueEventsWithOnePair++;
		if(isTruePassCuts) info_trueEventsPassingCuts++;
		else 		   info_trueEventsWithOnePairFailingCuts++;
		isTruePassCuts = true; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!! thake thi spair anyway !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	} // up to now we didn't write anything to the file

	// reset the cuts
	m_kin.setCuts(m_invMassCut, m_pTransverseCut, m_EtaCut, m_pTsystemCut, m_ySystemCut); // Set the cuts
	// reset the smeared mu+ and mu- vectors
	m_antiVec.clear();
	m_Vec.clear();
	// run AGAIN over all particles in the event record and fill the smeared vectors
	for (int i=0 ; i<(int)event.size() ; i++) {
		if (event[i].isFinal() && abs(event[i].id())==absId) { extract2vectors(event[i], doSmear); } // build the SMEARED m_Vec m_antiVec vectors
	} // if the boolean variable doSmear==false, then the overloaded extract2vectors(event[i], doSmear) method is identical to the extract2vectors(event[i]) method

        // check if the smeared pair passed the cuts:
	if ((int)m_Vec.size()+(int)m_antiVec.size() == 2) { // see if there is only one m+mu- smeared pair (if >1, reject the event)
		calculateXbjorken(event); // calculate bjorken X
		isSmearedPassCuts = m_kin.applyAllCuts(m_Vec[0], m_antiVec[0]); // see if the mu+mu- smeared pair pass all the cuts
		registerParticle(m_Vec[0], m_VecTmp[0], writeXbjorken, write2tree, isSmearedPassCuts);         // register the smeared mu-
		registerParticle(m_antiVec[0], m_antiVecTmp[0], writeXbjorken, write2tree, isSmearedPassCuts); // register the smeared mu+
		/*
		if(abs(m_Vec[0].eta())>m_EtaCut || abs(m_antiVec[0].eta())>m_EtaCut) {
			if(isSmearedPassCuts) cout << "eta cut=" << m_EtaCut << ", l.eta=" << m_Vec[0].eta() << ", al.eta=" << m_antiVec[0].eta() << endl;
		}
		if(m_Vec[0].pT()<m_pTransverseCut || m_antiVec[0].pT()<m_pTransverseCut) {
			if(isSmearedPassCuts) cout << "l.pT=" << m_Vec[0].pT() << ", al.pT=" << m_antiVec[0].pT() << endl;
		}
		*/
		if(!isSmearedPassCuts) {
			cout << m_eventNumber << ": rejected due to ";
			if(abs(m_Vec[0].eta())>m_EtaCut || abs(m_antiVec[0].eta())>m_EtaCut) { cout << "failed eta cut" << endl; }
			if(m_Vec[0].pT()<m_pTransverseCut || m_antiVec[0].pT()<m_pTransverseCut) { cout << "failed pT cut" << endl; }
			if(m_Vec[0].id()+m_antiVec[0].id()!=0) { cout << "one flipped charge" << endl; }
			if(m_kin.invMass(m_Vec[0], m_antiVec[0])<m_invMassCut) { cout << "failed mHat cut" << endl; }
		}
		if(m_Vec[0].id()+m_antiVec[0].id()==0) info_smearedEventsWithOnePair++;
		else 			               info_eventsWithFlippedCharge++;
		if(isSmearedPassCuts)                  info_smearedEventsPassingCuts++;
		else				       info_smearedEventsWithOnePairFailingCuts++;
	} // we now have one mu+mu- smeared pair registered 
	else info_eventsWithMoreThanOnePair++;

	if(isTruePassCuts) m_eventNumber ++; // increment the number of good "true" pairs

        return isTruePassCuts;
}




void extractFromEventRecord::finalizeOutput() {
	m_outFile.close();
}

void extractFromEventRecord::finalizeOutput(bool finalizeTree) {
	if (finalizeTree) {
	        if (m_fullRecord) {
			m_fullRecordTree->Print();
			m_fullRecordTree->Write();
			
			m_fullRecordAllPairsTree->Print();
                        m_fullRecordAllPairsTree->Write();

			m_fullRecordAllTruePairsTree->Print();
                        m_fullRecordAllTruePairsTree->Write();
		}
        	else {
			m_reducedRecordTree->Print();
			m_reducedRecordTree->Write();

			m_reducedRecordAllPairsTree->Print();
                        m_reducedRecordAllPairsTree->Write();
			
			m_reducedRecordAllTruePairsTree->Print();
                        m_reducedRecordAllTruePairsTree->Write();
		}

		m_treeFile->Write();
	}
}

}

