/************************************************************/
/* class to evaluate the user defined couplings             */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
#include "couplings.h"
using namespace std;

namespace Moses1 {

couplings::couplings(){}
//new code by avi 28/6
couplings::couplings(physicalParameters &physPrm)
{
	setFGBqQbarCouplings(physPrm);
}
couplings::~couplings(){}


// to return the couplings of a fermion with helicity lambda=+1/2 to Z0/gamma gauge bosons
double couplings::getCouplings(double helicity,
				pdtParticle &propagator, customParticle &fermion,
				physicalParameters &physPrm) {
	double g =0.;

	if (propagator.id == PDTGAMMA) {g = fermion.pdt_prt.charge;} // is photon ?
	else if (propagator.id == PDTZ) { // is Z0 boson ?
		if (helicity < 0.) { // for a fermion with negative helicity
			g = fermion.weakI3 - fermion.pdt_prt.charge*physPrm.sinThetaW2;
		}
		else { // for a fermion with positive helicity
			g = (-1.) * fermion.pdt_prt.charge * physPrm.sinThetaW2;
		}
		g *= 1./sqrt(physPrm.sinThetaW2*physPrm.cosThetaW2);
	}
	else {
		cerr << "error: in couplings.cxx: propogator with id=" << propagator.id << " is not supported" << endl;
		exit(-1);
	}
	return(g);
}


double couplings::getCouplings(unsigned int nMode, double helicity,
				pdtParticle &propagator, customParticle &fermion,
				physicalParameters &physPrm) {

	if (nMode == 0) {return getCouplings(helicity,propagator,fermion,physPrm);} // S.M couplings
	else {return sqrt(2)*getCouplings(helicity,propagator,fermion,physPrm);} // E.D: simply multiply by sqrt(2). See Tom Rizzo's paper
	//-------------------------------------------------------------------------------------------------------
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////// Avi New Code. ////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////Massiv Gloun /////////////////////////////
// cuppelings..

dcomplex  couplings::getFGBqQbarCouplings(pdtParticle &propagator, physicalParameters &, unsigned int Lambda_I,
								customParticle &qfermion, double qhelicity, customParticle &Qfermion, double Qhelicity)
{
	if (propagator.id !=  PDTGLU) 
		{// it is not the PDTGLU
			cout << "this is not a gloun in getMassiveGlounSchannelCouplings " << endl;
			return dcomplex(0.,0.);
		}
		
	if (Qhelicity=!qhelicity)
		{
			return dcomplex(0.,0.); 
		
		}
		
	return FGBVertexCouplingsArray[qhelicity<0?0:1][qfermion.pdt_prt.id-1][Qfermion.pdt_prt.id-1][Lambda_I-1];
}









///////////////////////////////////////////////////////
//* set FGBqQbarCouplings
///////////////////////////////////////////////////////
void   couplings::setFGBqQbarCouplings(physicalParameters &physPrm)
{
	int Lambda_I;
	//customParticle Qfermion;
	double FGBCouplingsQQbar1[18]={-0.2, -0.2, -0.2, -0.2,   1., 1.,
								   -0.2,   0., -0.2,   0., 0.2, 0., 
									 0., -0.2,   0., -0.2,  0.,  5.0
 							};
	gMatrix.SetGellMannMatrices();						
	//{PDTDWN (1) ,PDTUP(2),PDTSTR(3),PDTCHM(4),PDTBOT(5),PDTTOP(6),PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU}		
				        //1,2,3,4,5,6 
	int flavorIndex1[6]   = {0,0,1,1,2,2};	
	for (int i=0 ; i < 18; i ++)
	{
		couplings::FGBCouplingsQQbar[i]= FGBCouplingsQQbar1[i];
	}	
	for (int i=0 ; i < 6; i ++)
	{
		couplings::flavorIndex[i]=flavorIndex1[i];
	}
	for (int qfermion = 1;qfermion <=6 ;qfermion++)
	{
		for(int Qfermion = 1;Qfermion <= 6;Qfermion++)
		{
			for (double qhelicity = -0.5 ; qhelicity <= 0.5 ; qhelicity += 1.)
			{	
				double Qhelicity = qhelicity;
				for (Lambda_I = 1; Lambda_I < 9; Lambda_I ++)
				{ 
					couplings::FGBVertexCouplingsArray [Qhelicity<0?0:1][qfermion-1][Qfermion-1][Lambda_I-1]=
							getFGBqQbarCouplingsbyIndex(physPrm, Lambda_I, qfermion,  qhelicity,  Qfermion,  Qhelicity);
													
				}				
			}
		}
	}
		
}
















///////////////////////////////////////////////////////////////////////////////////////
// 	Print FGBqQbarCouplings
///////////////////////////////////////////////////////////////////////////////////////
void   couplings::printFGBqQbarCouplings()
{
	int Lambda_I;
	for (double qhelicity = -0.5 ; qhelicity <= 0.5 ; qhelicity += 1.)
	{	
		double Qhelicity = qhelicity;
		for (int qfermion = 1;qfermion <=6 ;qfermion++)
		{
			for(int Qfermion = 1;Qfermion <= 6;Qfermion++)
			{			
				cout<< " qfermion, Qfermion, qhelicity, "<<qfermion << " " <<Qfermion<<" "<< qhelicity<<" - ";
				for (Lambda_I = 1; Lambda_I < 9; Lambda_I ++)
				{
					cout<<abs(couplings::FGBVertexCouplingsArray [Qhelicity<0?0:1][qfermion-1][Qfermion-1][Lambda_I-1]) << " ";
					
				}	
				cout << endl;
			}
		}
	}
		
}










/////////////////////////////////////////////////////////////////////////////////////////////////////////
//get FGB qQbar CouplingsbyIndex
/////////////////////////////////////////////////////////////////////////////////////////////////////////
// qfermion and Qfermion posible values are: 1,2,3,4,5,6  not 0!
dcomplex  couplings::getFGBqQbarCouplingsbyIndex(physicalParameters &physPrm, unsigned int Lambda_I,
								int qfermion, double qhelicity, int Qfermion, double Qhelicity)

{
	if (qhelicity != Qhelicity)
	{	//no interaction
		return dcomplex(0.,0.);
	}
	// they have the same helicity ...
	
	int qId			 = abs (qfermion)-1; // the paritcle ID (as in 1 for PDTDWN)  //array start from 0 and not from 1
	int QId			 = abs (Qfermion)-1;
	
//	int qflavorIndex = flavorIndex[qId]; //the index required by gel mann matrix
//	int QflavorIndex = flavorIndex[QId];
	int qflavorIndex = qId << 1; //the index required by gel mann matrix
	int QflavorIndex = QId << 1;

	double helicity  = qhelicity; 			// Same Helicity Particles
	gMatrix.SetGellMannMatrices();
	int qOffset=-1; 							// pdt 1 =1 and not 0.. so we need to take (index = qfermion.pdt_prt.id - 1) 
	int QOffset=-1; 	
	// pdt 1 =1 and not 0.. so we need to take (index = qfermion.pdt_prt.id - 1) 
	if (helicity < 0.) 
	{//left handed group
		// qOffset = 0;  // and also same Halicity...
				//cout << "here left" ;
		return physPrm.g_mGluonStar*gMatrix.Lambda[Lambda_I][qflavorIndex][QflavorIndex] * sqrt(abs(couplings::FGBCouplingsQQbar[qId]*couplings::FGBCouplingsQQbar[QId]));
		
	}	
	//Else its right handed : ie  > 0.	
	//chaking if on the same group:	
	//cout << " right " ;
	qOffset = ((qId % 2) == 0) ?6: 12;	//a > b ? a : b
	QOffset = ((QId % 2) == 0) ?6: 12;	
	if  (qOffset != QOffset)
	{ //not in the same group.
		return dcomplex(0.,0.);
	}	
			
	/*		if ((qflavorIndex == 2)&&(QflavorIndex==2)&&(Lambda_I==8)){
				cout << "*******************************************************************************";
				cout << endl;
				cout << "Qfermion "<< Qfermion<<endl;
			cout <<" " << physPrm.g_mGluonStar*gMatrix.Lambda[Lambda_I][qflavorIndex][QflavorIndex] <<" ";
			cout << " " << sqrt(abs(couplings::FGBCouplingsQQbar[qId+qOffset]*couplings::FGBCouplingsQQbar[QId+QOffset]));
			//exit(0);
			}*/
	return physPrm.g_mGluonStar*gMatrix.Lambda[Lambda_I][qflavorIndex][QflavorIndex]*sqrt(abs(couplings::FGBCouplingsQQbar[qId+qOffset]*couplings::FGBCouplingsQQbar[QId+QOffset]));
	
}
//////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////// get FGBgroup ///////////////////////////////////////
int couplings::getFGBgroup(	customParticle &qfermion, double qhelicity, customParticle &Qfermion, double Qhelicity)
{
	int upRight[]={0,1,0,1,0,1};
	int downRight[]={1,0,1,0,1,0};
	if (qhelicity != Qhelicity)
	{
		return -1;
	}
	if (qhelicity < 0.)
	{
		return 0;
	}
	//else right handed
	if (downRight[qfermion.pdt_prt.id-1]*downRight[Qfermion.pdt_prt.id-1])
	{	
		return 1;
	}
	if (upRight[qfermion.pdt_prt.id-1]*upRight[Qfermion.pdt_prt.id-1])
	{	
		return 2;
	}
	//Group = 2; //Up type right ;
	//Group = 1; //Down type right ;
	//Group = 0; //Left group
	//Group =-1 no group
	//no group was found;
	return -1;	
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////old code (working)
dcomplex  couplings::getMassiveGlounSchannelCouplings(unsigned int Lambda_I,unsigned int nMode, double helicity1,double helicity2,
                                        pdtParticle &propagator,  customParticle &fermion1,customParticle &fermion2,
                                        physicalParameters &physPrm){
    dcomplex  g (0.,0.);
//    double gScaling = 0.99 ; // this is the relation between gGlounStar/gQCD

        gMatrix.SetGellMannMatrices();
  //  customParticle fermion2 = fermion1;  //schannel

    unsigned int fermion_1_Index = 0;
    unsigned int fermion_2_Index = 0;
    //gMatrix.gellMannMatrices();
    double uptaype1  = 0.; // false the particle is not up type
    double uptaype2  = 0.; // false the particle is not up type

    if (fermion1.pdt_prt.id == PDTUP)  {fermion_1_Index = 0; uptaype1 = 1.;} //u
    if (fermion1.pdt_prt.id == PDTDWN) {fermion_1_Index = 0; uptaype1 = -1.;} //d
    if (fermion1.pdt_prt.id == PDTCHM) {fermion_1_Index = 1; uptaype1 = 1.;} //c
    if (fermion1.pdt_prt.id == PDTSTR) {fermion_1_Index = 1; uptaype1 = -1.;} //s
    if (fermion1.pdt_prt.id == PDTBOT) {fermion_1_Index = 2; uptaype1 = 1.;} //t
    if (fermion1.pdt_prt.id == PDTTOP) {fermion_1_Index = 2; uptaype1 = -1.;} //b

    if (fermion2.pdt_prt.id == PDTUP)  {fermion_2_Index = 0; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTDWN) {fermion_2_Index = 0; uptaype2 = -1.;}
    if (fermion2.pdt_prt.id == PDTCHM) {fermion_2_Index = 1; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTSTR) {fermion_2_Index = 1; uptaype2 = -1.;}
    if (fermion2.pdt_prt.id == PDTBOT) {fermion_2_Index = 2; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTTOP) {fermion_2_Index = 2; uptaype2 = -1.;}
	
	if (uptaype1 != uptaype2){
		return dcomplex(0,0);; 
	}

   if (propagator.id ==  PDTGLU) {
                //cout << "this a gloun in getMassiveGlounSchannelCouplings " << endl; works!
		if (helicity1 == helicity2)
		{//  for s channel, need revisite for t channel.  assume f1 = f2.
                if ((fermion1.pdt_prt.id != PDTTOP)||(fermion1.pdt_prt.id != PDTBOT)){
					g = -0.2 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];  // 0.2 = 1/5 , fermion that are not top nor bottom
                }
				if(helicity1 < 0.)
				{// helicity1 < 0 --  left handed particle----------------------------------------------------------
					if (fermion1.pdt_prt.id == PDTTOP)
					{//its top
						g = 1.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
						//cout << "here 1";
                    }//if (fermion1.pdt_prt.id == PDTTOP)
                    else
					if (fermion1.pdt_prt.id == PDTBOT)
					{//its bottom
						g = 1.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index]; 
                    }//if (fermion1.pdt_prt.id == PDTBOT)
                 }//(helicity1 < 0.)-------------------------------------------------------------------------
                 else 
				 if (helicity1 > 0.)
				 { 
				 // right handed particle-----------------------------------------------------------
							if ( fermion1.pdt_prt.id == PDTTOP )
							{// its right t
                            // down type * uptaype2 = -1 if the particles are from different groups. Q_Ur and Q_Dr
								g =  5.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
							//	cout << "here 5";
							//	cout <<"Lambda_I "<<Lambda_I<<" t tbar state up type " << gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index]<<" g= "<< g << endl;
                             }
                             else 
							 if (fermion1.pdt_prt.id == PDTBOT)
							 {//its right b
                                  g =  0.2 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
                             }                   
                }//(helicity1 > 0.)
			}//(helicity1 == helicity2)
			g = g * physPrm.g_mGluonStar;// * physPrm.gQCD;		
			//cout << "physPrm.g_mGluonStar "<<physPrm.g_mGluonStar<<endl;
            return(g);
        }else 
		{// propagator.id ==  PDTGLU
			cout << "this is not a gloun in getMassiveGlounSchannelCouplings " << endl;
		}



        return 0.;
} //couplings::getCouplings



}

