/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
#include "diLeptonBasicAnalyzer.h"

using namespace std;

namespace Moses1 {

dileptonAnalyzer::dileptonAnalyzer()
{
	m_doSmearing = false;
}

dileptonAnalyzer::~dileptonAnalyzer(){}


void dileptonAnalyzer::setSmear(double average, double sigma, double resolution, double pTTrackScale, bool printChanges){
	m_smearing.settings(average, sigma, resolution, pTTrackScale);
	m_doSmearing = true;
	m_printChanges = printChanges;
}

void dileptonAnalyzer::setTrue(bool doTrue) {
	m_doTrue = doTrue;
}

void dileptonAnalyzer::setCuts(double invMassCut, double pTransverseCut, double EtaCut) {
        m_invMcut = invMassCut;
        m_pTcut = pTransverseCut;
        m_etacut = EtaCut;
}



void dileptonAnalyzer::clearAllLeptons() {
	if (m_allgoodLeptons.size() != 0) { m_allgoodLeptons.clear(); }
	if (m_allgoodaLeptons.size() != 0) { m_allgoodaLeptons.clear(); }
	if (m_allgoodLeptonsTrue.size() != 0) { m_allgoodLeptonsTrue.clear(); }
        if (m_allgoodaLeptonsTrue.size() != 0) { m_allgoodaLeptonsTrue.clear(); }
}



int dileptonAnalyzer::getAllLeptons(vector<fullRecord> &allgoodLeptons) {
	allgoodLeptons = m_allgoodLeptons;
	return (int)allgoodLeptons.size();
}
int dileptonAnalyzer::getAllaLeptons(vector<fullRecord> &allgoodaLeptons) {
        allgoodaLeptons = m_allgoodaLeptons;
        return (int)allgoodaLeptons.size();
}



int dileptonAnalyzer::getAllLeptonsTrue(vector<fullRecord> &allgoodLeptonsTrue) {
        allgoodLeptonsTrue = m_allgoodLeptonsTrue;
        return (int)allgoodLeptonsTrue.size();
}
int dileptonAnalyzer::getAllaLeptonsTrue(vector<fullRecord> &allgoodaLeptonsTrue) {
        allgoodaLeptonsTrue = m_allgoodaLeptonsTrue;
        return (int)allgoodaLeptonsTrue.size();
}

int dileptonAnalyzer::loopOverEvents(string filePath) {
	ifstream file;
	file.open(filePath.c_str());
	m_events = 0;
	clearAllLeptons();
	while(!file.eof()) {
		m_Nlept = m_Nalept = 0;
		// read number of leptons and antileptons in the event:
                file >> m_Nlept >> m_Nalept;
		if (m_Nlept+m_Nalept == 0) {break;}
		else if (m_Nlept+m_Nalept > 0) {
			m_events++;
			if (m_Lepton.size() != 0) { m_Lepton.clear(); }
			if (m_aLepton.size() != 0) { m_aLepton.clear(); }
			// loop over all leptons and antileptons in the event:
			for(int i=1 ; i<=(m_Nlept+m_Nalept) ; i++) {
				readParticleEntry(file, m_tmpLepton);
				if (m_tmpLepton.id > 0) { m_Lepton.push_back(m_tmpLepton); }
				else if(m_tmpLepton.id < 0) { m_aLepton.push_back(m_tmpLepton); }
				else { cout << "error in id" << endl;    exit(-1); }
			}
			// apply the cuts and save good pairs in 2 vectors
			for (int k=0 ; k<m_Nlept ; k++) {
				for (int j=0 ; j<m_Nalept ; j++) {
					if ( invMcut(m_Lepton[k],m_aLepton[j]) ) {
						if ( pTcut(m_Lepton[k],m_aLepton[j]) ) {
							if ( etaCut(m_Lepton[k],m_aLepton[j]) ) {
								m_allgoodLeptons.push_back(m_Lepton[k]);
								m_allgoodaLeptons.push_back(m_aLepton[j]);
							}
						}
					}
				}
			}
		} // end of m_Nlept+m_Nalept > 0
	} // end of while(!file.eof())
	file.close();
	return m_events;
}



int dileptonAnalyzer::loopOverEvents(string filePath, string treeName, int nEvents, int nStart) {

	m_treeFile = new TFile(filePath.c_str());
	m_dsigmadmHatTree = (TTree*)m_treeFile->Get(treeName.c_str());

	cout << "looking in file: " << filePath << "  for a tree called: " << treeName << endl;

	// set branches
        m_dsigmadmHatTree->SetBranchAddress("eventNumber",&m_eventNumber);
        m_dsigmadmHatTree->SetBranchAddress("id",&m_id);
        m_dsigmadmHatTree->SetBranchAddress("m",&m_m);
        m_dsigmadmHatTree->SetBranchAddress("e",&m_e);
	m_dsigmadmHatTree->SetBranchAddress("px",&m_px);
	m_dsigmadmHatTree->SetBranchAddress("py",&m_py);
	m_dsigmadmHatTree->SetBranchAddress("pz",&m_pz);
	m_dsigmadmHatTree->SetBranchAddress("pAbs",&m_pAbs);
	m_dsigmadmHatTree->SetBranchAddress("pT",&m_pT);
	m_dsigmadmHatTree->SetBranchAddress("pPlus",&m_pPlus);	
	m_dsigmadmHatTree->SetBranchAddress("pMinus",&m_pMinus);
	m_dsigmadmHatTree->SetBranchAddress("theta",&m_theta);
	m_dsigmadmHatTree->SetBranchAddress("phi",&m_phi);
	m_dsigmadmHatTree->SetBranchAddress("y",&m_y);
	m_dsigmadmHatTree->SetBranchAddress("eta",&m_eta);

	m_dsigmadmHatTree->SetBranchAddress("xq",&m_xq);
	m_dsigmadmHatTree->SetBranchAddress("xqbar",&m_xqbar);


	treeName += "_True";
	m_tru_dsigmadmHatTree = (TTree*)m_treeFile->Get(treeName.c_str());
	
	m_tru_dsigmadmHatTree->SetBranchAddress("eventNumber",&m_tru_eventNumber);
        m_tru_dsigmadmHatTree->SetBranchAddress("id",&m_tru_id);
        m_tru_dsigmadmHatTree->SetBranchAddress("m",&m_tru_m);
        m_tru_dsigmadmHatTree->SetBranchAddress("e",&m_tru_e);
        m_tru_dsigmadmHatTree->SetBranchAddress("px",&m_tru_px);
        m_tru_dsigmadmHatTree->SetBranchAddress("py",&m_tru_py);
        m_tru_dsigmadmHatTree->SetBranchAddress("pz",&m_tru_pz);
        m_tru_dsigmadmHatTree->SetBranchAddress("pAbs",&m_tru_pAbs);
        m_tru_dsigmadmHatTree->SetBranchAddress("pT",&m_tru_pT);
        m_tru_dsigmadmHatTree->SetBranchAddress("pPlus",&m_tru_pPlus);
        m_tru_dsigmadmHatTree->SetBranchAddress("pMinus",&m_tru_pMinus);
        m_tru_dsigmadmHatTree->SetBranchAddress("theta",&m_tru_theta);
        m_tru_dsigmadmHatTree->SetBranchAddress("phi",&m_tru_phi);
        m_tru_dsigmadmHatTree->SetBranchAddress("y",&m_tru_y);
        m_tru_dsigmadmHatTree->SetBranchAddress("eta",&m_tru_eta);

	// get tree size
	Int_t treeNentries = (Int_t)m_dsigmadmHatTree->GetEntries();
	
	Int_t maxEnrties = treeNentries;
	if(nEvents > 0) maxEnrties = (Int_t)nEvents;

	m_events = 0;
	m_goodLeptons = 0;
	clearAllLeptons();
	
	Int_t currentEventNumber;
	Int_t i=(Int_t)nStart;
	Int_t j;

	bool isOneGoodPair = false;	

	cout << "start=" << i << ", events=" << maxEnrties << endl;

	while ( i < (maxEnrties+(Int_t)nStart) )
	{
		//count the number of leptons and antileptons in the current event
		m_Nlept = m_Nalept = 0;
		j=1;
		m_dsigmadmHatTree->GetEntry(i);
		currentEventNumber = (Int_t)m_eventNumber;
		while (m_eventNumber == currentEventNumber) {
			if (m_id>0) {m_Nlept++;}
			else {m_Nalept++;}
			if ((i+j) < (maxEnrties+(Int_t)nStart)  && (i+j)<treeNentries) {
				m_dsigmadmHatTree->GetEntry(i+j);
				j++;
			}
			else {break;}
		}
		//----------------------------------------------------------------

		if (m_Nlept+m_Nalept == 0) {break;}
		else if (m_Nlept+m_Nalept > 0) {m_events++;}
		if (m_Nlept+m_Nalept == 2) {
			// clear vectors if not already cleared
			if (m_Lepton.size() != 0) { m_Lepton.clear(); }
			if (m_aLepton.size() != 0) { m_aLepton.clear(); }
			if (m_LeptonTrue.size() != 0) { m_LeptonTrue.clear(); }
                        if (m_aLeptonTrue.size() != 0) { m_aLeptonTrue.clear(); }

			//treeNumberMatch = lastVisitedTreeNumber;
			// loop over all leptons and antileptons in the event:
			for(Int_t l=i ; l<=(i + m_Nlept+m_Nalept) ; l++) {
				//---------read the leptons of the current event----------
				readParticleEntry(l, m_tmpLepton);
				if(m_doTrue) {
					readTrueParticleEntry(l, m_tmpLeptonTrue);
					if(m_tmpLepton.eventNumber != m_tmpLeptonTrue.eventNumber) cout << "warning: m_tmpLepton.eventNumber != m_tmpLeptonTrue.eventNumber" << endl;
				}
				//--------------------------------------------------------
				if (m_tmpLepton.id > 0) {
					m_Lepton.push_back(m_tmpLepton);
					if(m_doTrue) m_LeptonTrue.push_back(m_tmpLeptonTrue);
				}
				else if(m_tmpLepton.id < 0) {
					m_aLepton.push_back(m_tmpLepton);
					if(m_doTrue) m_aLeptonTrue.push_back(m_tmpLeptonTrue);
				}
				else { cout << "error in id" << endl;    exit(-1); }
			}
			// apply the cuts and save good pairs in 2 vectors
			for (int k=0 ; k<m_Nlept ; k++) {
				for (int j=0 ; j<m_Nalept ; j++) {
					if ( invMcut(m_Lepton[k],m_aLepton[j]) ) {
						if ( pTcut(m_Lepton[k],m_aLepton[j]) ) {
							if ( etaCut(m_Lepton[k],m_aLepton[j]) ) {
								m_allgoodLeptons.push_back(m_Lepton[k]);
								m_allgoodaLeptons.push_back(m_aLepton[j]);
								if(m_doTrue) m_allgoodLeptonsTrue.push_back(m_LeptonTrue[k]);
								if(m_doTrue) m_allgoodaLeptonsTrue.push_back(m_aLeptonTrue[j]);
								isOneGoodPair = true;
								m_goodLeptons++;
							}
						}
					}
					if(isOneGoodPair) break;
				}
				if(isOneGoodPair) break;
			}
		} // end of m_Nlept+m_Nalept > 0
		i += (m_Nlept+m_Nalept);
		if(m_events%1==0) cout << "events=" << m_events << "\r" << flush;
	} // end of while(i<treeNentries)
	m_treeFile->Close();
	delete m_treeFile;
	return m_events;
}


int dileptonAnalyzer::allGoodLeptons(){
	return m_goodLeptons;
}


void dileptonAnalyzer::readParticleEntry(ifstream &file, fullRecord &fulRec) {
	file >> fulRec.id;
	file >> fulRec.m;
	file >> fulRec.e >> fulRec.px >> fulRec.py >> fulRec.pz;
	file >> fulRec.pAbs >> fulRec.pT >> fulRec.pPlus >> fulRec.pMinus;
	file >> fulRec.theta >> fulRec.phi;
	file >> fulRec.y >> fulRec.eta;
}

void dileptonAnalyzer::readParticleEntry(Int_t l, fullRecord &fulRec) {
	m_dsigmadmHatTree->GetEntry(l);

	fulRec.eventNumber = m_eventNumber;
	fulRec.id = m_id;
	fulRec.m = m_m;
	fulRec.e = m_e;
	fulRec.px = m_px;
	fulRec.py = m_py;
	fulRec.pz = m_pz;
	fulRec.pAbs = m_pAbs;
	fulRec.pT = m_pT;
	fulRec.pPlus = m_pPlus;
	fulRec.pMinus = m_pMinus;
	fulRec.theta = m_theta;
	fulRec.phi = m_phi;
	fulRec.y = m_y;
	fulRec.eta = m_eta;
	
	fulRec.xq = m_xq;
	fulRec.xqbar = m_xqbar;

	if(m_doSmearing)
        {
                double pTSmeared = m_smearing.pTsmear( m_pT );
                double pxSmeared = m_smearing.pT2px(pTSmeared, m_phi);
                double pySmeared = m_smearing.pT2py(pTSmeared, m_phi);
                double pzSmeared = m_smearing.pT2pz(pTSmeared, m_theta);
                double eSmeared  = m_smearing.pT2e(pTSmeared, m_theta);

                // pring the changes
                if(m_printChanges)
                {
                        m_smearing.compare2true( m_px, m_py, m_pz, m_e );
                        if(pTSmeared<0) {cout << "smeared pT=" << pTSmeared  << " ==> charge flip !" << endl;}
                }

                // update the smeared four momentum in the particle
                fulRec.px = pxSmeared;
                fulRec.py = pySmeared;
                fulRec.pz = pzSmeared;
                fulRec.e  = eSmeared;
		fulRec.pT = pTSmeared;
                // flip the charge and the id due to resolution
                //int idOrig = fulRec.id;
                if(pTSmeared<0)
                {
                        fulRec.id *= -1;
                }

        }
}


void dileptonAnalyzer::readTrueParticleEntry(Int_t l, fullRecord &fulRec) {
        m_tru_dsigmadmHatTree->GetEntry(l);

	fulRec.eventNumber = m_tru_eventNumber;
        fulRec.id = m_tru_id;
        fulRec.m = m_tru_m;
        fulRec.e = m_tru_e;
        fulRec.px = m_tru_px;
        fulRec.py = m_tru_py;
        fulRec.pz = m_tru_pz;
        fulRec.pAbs = m_tru_pAbs;
        fulRec.pT = m_tru_pT;
        fulRec.pPlus = m_tru_pPlus;
        fulRec.pMinus = m_tru_pMinus;
        fulRec.theta = m_tru_theta;
        fulRec.phi = m_tru_phi;
        fulRec.y = m_tru_y;
        fulRec.eta = m_tru_eta;
}



void dileptonAnalyzer::readParticleEntry(ifstream &file, minimalRecord &minRec) {
	file >> minRec.id;
	file >> minRec.m;
	file >> minRec.e >> minRec.px >> minRec.py >> minRec.pz;
}

double dileptonAnalyzer::invMass(fullRecord &lepton, fullRecord &alepton) {
	return	(
		    sqrt( lepton.m*lepton.m + alepton.m*alepton.m
			  + 2 * lepton.e*alepton.e
			  - 2 * (lepton.px*alepton.px + lepton.py*alepton.py + lepton.pz*alepton.pz)
		        )
		);
}

double dileptonAnalyzer::pT(fullRecord &lepton, fullRecord &alepton) {
	return  (
		   sqrt( (lepton.px + alepton.px)*(lepton.px + alepton.px)
			 + (lepton.py + alepton.py)*(lepton.py + alepton.py)
		       )
                );
}

double dileptonAnalyzer::y(fullRecord &lepton, fullRecord &alepton) {
	return	(
		   0.5*log(   (lepton.e + alepton.e + lepton.pz + alepton.pz)
			    / (lepton.e + alepton.e - lepton.pz - alepton.pz)
			  )
		);
}

double dileptonAnalyzer::cosThetaBoost(fullRecord &lepton, fullRecord &alepton) { // calculate cos(theta) relative to the boost in the Helicity frame
	//----- use pythia's methods:
	// The Particle object contains a Vec4 p that stores the particle four-momentum.
	// The Vec4 class gives an implementation of four-vectors.
	// The member function names are based on the assumption that these represent momentum vectors.
	// Thus one can get or set px(), py(), pz() and e(), but not x, y, z or t.
	//-----------------
	m_p1.px(lepton.px);
	m_p2.px(alepton.px);
	//-----------------
	m_p1.py(lepton.py);
	m_p2.py(alepton.py);
	//-----------------
	m_p1.pz(lepton.pz);
	m_p2.pz(alepton.pz);
	//-----------------
	m_p1.e(lepton.e);
	m_p2.e(alepton.e);
	//-----------------
	m_pSum = m_p1 + m_p2;
	//-----------------
	m_p1.bstback(m_pSum);	// boost p1 from lab frame, back to c.m.f with beta = -p/E
	m_p2.bstback(m_pSum);	// boost p2 from lab frame, back to c.m.f with beta = -p/E

        return(
		( m_p1.px()*m_pSum.px() + m_p1.py()*m_pSum.py() + m_p1.pz()*m_pSum.pz() ) /
		(
			sqrt(m_p1.px()*m_p1.px() + m_p1.py()*m_p1.py() + m_p1.pz()*m_p1.pz()) *
			sqrt(m_pSum.px()*m_pSum.px() + m_pSum.py()*m_pSum.py() + m_pSum.pz()*m_pSum.pz())
		)	
        );
}

double dileptonAnalyzer::cosThetaCollinsSoper(fullRecord &lepton, fullRecord &alepton) { // calculate cos(theta) relative to y in CS frame
	double invMassSquare = invMass(lepton,alepton)*invMass(lepton,alepton);
	double pTsquare = pT(lepton,alepton)*pT(lepton,alepton);
	double cosThetaCS = 2.*( lepton.pz*alepton.e - lepton.e*alepton.pz ) /
				sqrt( invMassSquare * (invMassSquare + pTsquare) );
	if (y(lepton,alepton) < 0) { cosThetaCS *= -1.; } // reclassify
	return cosThetaCS;
}

double dileptonAnalyzer::cosThetaCollinsSoper(fullRecord &lepton, fullRecord &alepton, bool quarkDirection) { // calculate cos(theta) relative to the quark direction in CS frame
        double invMassSquare = invMass(lepton,alepton)*invMass(lepton,alepton);
        double pTsquare      = pT(lepton,alepton)*pT(lepton,alepton);
        double cosThetaCS    = 2.*( lepton.pz*alepton.e - lepton.e*alepton.pz ) /
                                  sqrt( invMassSquare * (invMassSquare + pTsquare) );
	double boostSign     = (lepton.pz + alepton.pz)/abs(lepton.pz + alepton.pz);
        if (lepton.xq < alepton.xqbar) { cosThetaCS *= -1.*boostSign; } // reclassify
	else { cosThetaCS *= +1.*boostSign; } // reclassify
        return cosThetaCS;
}


bool dileptonAnalyzer::invMcut(fullRecord &lepton, fullRecord &alepton) {
	if (invMass(lepton,alepton) >= m_invMcut) { return true; }
	else { return false; }
}

bool dileptonAnalyzer::pTcut(fullRecord &lepton, fullRecord &alepton) {
	if (lepton.pT >= m_pTcut   &&   alepton.pT >= m_pTcut) { return true; }
	else { return false; }
}

bool dileptonAnalyzer::etaCut(fullRecord &lepton, fullRecord &alepton) {
	/*
	if (lepton.eta < m_etacut   &&   lepton.eta > -1.*m_etacut) {
		if (alepton.eta < m_etacut   &&   alepton.eta > -1.*m_etacut) { return true; }
		else { return false; }
	}
	else { return false; }
	*/
	if (abs(lepton.eta)<m_etacut && abs(alepton.eta)<m_etacut) { return true; }
	else { return false; }
}

}

