#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TObject.h"
#include "TString.h"

#include <map>

using namespace std;

TFile* f = NULL;
TTree* controlTree = NULL;


//---------------------------------- point it to where you want -------------------------------------
TString sRootDictDir  = "/srv01/tau/hod/moses-1.2.0/rootdict/";                      // mandatory
//TString sRootDictDir = gSystem->ExpandPathName("$MOSESSYS/rootdict/");             // another option
TString sDataFile = "/srv01/tau/hod/moses-1.2.0/data/mosesRunRecord.root";           // mandatory
//TString sDataFile = gSystem->ExpandPathName("$MOSESSYS/data/mosesRunRecord.root"); // another option
#include "/srv01/tau/hod/moses-1.2.0/rootdict/mapTObj.C"                             // mandatory
//----------------------------------------------------------------------------------------------------


void loadLibrary()
{
	gSystem->Load(sRootDictDir+"librootdict.so");
}


void readTree()
{
	TFile *f = new TFile(sDataFile);
	
	// there is only one important master tree...
	controlTree = (TTree *)f->Get("runControl/controlTree");
	
	// now make friends of the master tree
	//controlTree->AddFriend("hardProcess/hardProcessTree", f);
	
	f->ls();
}



//------------------------------- help methods -------------------------------
bool isKeyFound(TMapii* mapii, int key)
{
	bool isFound = true;
	if(key>0) isFound = (mapii->find(key)==mapii->end()) ? false:true;
	//if(!isFound) cout << "key=" << key << " was not found" << endl;
	return isFound;
}

int getCorrectedKey(TMapii* mapii, int key)
{
	if (key<=0) return key;
	int correctedKey = ( !isKeyFound(mapii,key) ) ? -1:key;
	return correctedKey;
}

int getVal(TMapii* mapii, int key, vector<int>* id)
{
	if(key<=0) return 0;
	else
	{
		int mosesIndex = (*mapii)[key];
		//int val = (key==0) ? 0:(*id)[mosesIndex];
		int val = (*id)[mosesIndex];
		val     = ( !isKeyFound(mapii,key) ) ? 0:val;
		return val;
	}
	return 0;
}
//-----------------------------------------------------------------------------



void eventLoop()
{
	loadLibrary();
	readTree();

	// controls
	int nMuons;
	controlTree->SetBranchAddress("Nmuons", &nMuons);


	// hard process map between pythia and moses event record index 
	mapTObj* pythia2mosesIndexMap = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.pythia2mosesIndexMap", &pythia2mosesIndexMap);
	
	vector<int>* id = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.id", &id);
	vector<int>* momA = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.momA", &momA);
	vector<int>* momB = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.momB", &momB);
	vector<int>* sonA = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.sonA", &sonA);
	vector<int>* sonB = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.sonB", &sonB);
	vector<int>* statusLevel = 0;
	controlTree->SetBranchAddress("hardProcess/hardProcessTree.statusLevel", &statusLevel);


	for (int i=0 ; i</*controlTree->GetEntries()*/1 ; i++)
	{
		// get the controls:
		controlTree->GetEntry(i);

		cout << "entry = " << i << ", size = " << pythia2mosesIndexMap->mapP->size() << endl;

		cout << "----------------------------------------------------------------------------------------------------------------------------------" << endl;
		cout << "statusLevel"
			 << "           " << "index (id)"
			 << "           " << "Mother A (id)"
			 << "           " << "Mother B (id)"
			 << "           " << "Daughter A (id)"
			 << "         " << "Daughter B (id)" << endl;
		cout << "----------------------------------------------------------------------------------------------------------------------------------" << endl;

		for(TMapii::iterator it=pythia2mosesIndexMap->mapP->begin() ; it!=pythia2mosesIndexMap->mapP->end() ; ++it)
		{
			int pythiaIndex  = it->first;
			int mosesIndex   = it->second;
			int associatedId = (*id)[ mosesIndex ];
		
			cout << "     " << (*statusLevel)[ mosesIndex ];
			cout << "\t\t\t";
			
			cout << pythiaIndex << " (" << associatedId << ")";
			cout << "\t\t\t";
			
			pythiaIndex  = (*momA)[ mosesIndex ];
			pythiaIndex  = getCorrectedKey( pythia2mosesIndexMap->mapP, pythiaIndex );
			associatedId = getVal(pythia2mosesIndexMap->mapP, pythiaIndex, id);
			cout << pythiaIndex << " (" << associatedId << ")";
			cout << "\t\t\t";
			
			pythiaIndex  = (*momB)[ mosesIndex ];
			pythiaIndex  = getCorrectedKey( pythia2mosesIndexMap->mapP, pythiaIndex );
			associatedId = getVal(pythia2mosesIndexMap->mapP, pythiaIndex, id);
			cout << pythiaIndex << " (" << associatedId << ")";
			cout << "\t\t\t";
			
			pythiaIndex  = (*sonA)[ mosesIndex ];
			pythiaIndex  = getCorrectedKey( pythia2mosesIndexMap->mapP, pythiaIndex );
			associatedId = getVal(pythia2mosesIndexMap->mapP, pythiaIndex, id);
			cout << pythiaIndex << " (" << associatedId << ")";
			cout << "\t\t\t";
			
			pythiaIndex  = (*sonB)[ mosesIndex ];
			pythiaIndex  = getCorrectedKey( pythia2mosesIndexMap->mapP, pythiaIndex );
			associatedId = getVal(pythia2mosesIndexMap->mapP, pythiaIndex, id);
			cout << pythiaIndex << " (" << associatedId << ")"
			<< endl;
		}
		cout << "----------------------------------------------------------------------------------------------------------------------------------" << endl;
	}
}
