#ifndef MOSESEVENT_h
#define MOSESEVENT_h

#include "mosesCommon.h"
#include "useHEPPDT.h"
#include "mosesJetFinderWrap.h"
#include "mosesParticleRecord.h"
#include "mosesJetRecord.h"
#include "mosesHardProcessRecord.h"
#include "mosesRunSummary.h"


using namespace std;
using namespace Pythia8;


namespace Moses1 {

class mosesEvent
{
	public:
		mosesEvent(TFile* file, vector<string>& particleRecordNames);
		~mosesEvent();
		
		void giveLHAPDFset(string sLHAPDFset);
		
		void passJetFinderWrappers(mosesClusterJetWrap* clusterJetWrap,
								   mosesCellJetWrap* cellJetWrap,
								   mosesFastJetWrap* fastJetWrap);
		
		bool fillAllParticleRecords(Particle &particle, bool isHardProcess);
		bool fillAllJetRecords(Event& event);
		bool fillAllRecordTrees();
		bool clearAllParticleRecords();
		bool writeAllRecordTrees();
		bool writeControlTree();
		
		bool makeAllTreesFriends();

		bool isHardProcessStatus(int status);
		bool isHardProcess(Event& event, int index);

		bool loopOverEvents(int evts, int printMod, Pythia& pythia);

	private:
		void fillSupportedParticles();
		void fillSupportedJets();

		void pickRecords();
		void buildControlTree();

		bool fillControlTree();
		bool resetControls();


		// general objects
		Pythia m_pythia;
		TFile* m_file;
		map<string, int> m_supportedParticles;
		map<string, int> m_supportedJets;
		vector<string>   m_particleRecordNames;
		vector<string>   m_jetRecordNames;
		bool m_bStatus;
		int m_evts;
		vector<int> m_vMothersList;
		TDirectory* m_dir;
		TTree* m_controlTree;
		string m_sLHAPDFset;
		
		mosesRunSummary* m_runSummary;
		
		// maps for the individual particle / jets operations
		map<string, mosesParticleRecord*> m_particlesMap;
		mosesClusterJetRecord* m_clusterJetRecord;
		mosesCellJetRecord*    m_cellJetRecord;
		mosesFastJetRecord*    m_fastJetRecord;
		mosesHardProcessRecord* m_hardProcessRecord;
		mosesClusterJetWrap* m_clusterJetWrap;
		mosesCellJetWrap*    m_cellJetWrap;
		mosesFastJetWrap*    m_fastJetWrap;
		map<string, bool> m_bRecord;
		map<string, int>  m_nRecords;
		int m_nHardPorcessParticles;
};


}

#endif
