#ifndef USELHAPDF_h
#define USELHAPDF_h
/************************************/
/* AN INTERFACE CLASS TO LHAPDFWrap */
/* class to evaluate the LHA PDFs   */
/************************************/
//#include "LHAPDFWrap.h"
#include "LHAPDF/LHAPDF.h"
#include "mosesCommon.h"
using namespace std;

namespace Moses1 {


enum lhaParton {
	LHADWN=1, LHAUP=2, LHASTR=3, LHACHM=4, LHABOT=5, LHATOP=6,
	LHADWNBAR=-1, LHAUPBAR=-2, LHASTRBAR=-3, LHACHMBAR=-4, LHABOTBAR=-5, LHATOPBAR=-6,
	LHAGLU=0
};


class lhapdf {
	public:
		lhapdf();
		~lhapdf();
		//index:  -6    -5    -4    -3    -2    -1    0   1 2 3 4 5 6
		//parton: tbar  bbar  cbar  sbar  ubar  dbar  glu d u s c b t
		
		// In an "all" partons call:
		//-------------------------
		// partonsxpdfx is a vector with index 0 <= i <= 12.
		// 0..5 = tbar, ..., ubar, dbar; 
		// 6 = g; 
		// 7..12 = d, u, ..., t

		// In an single parton call:
		//-------------------------
		// partonsxpdfx is a vector with a SINGLE index i=0.
		// the flavors corresponding integers are as in the LHAPDF manual:
		// -6..-1 = tbar,...,ubar, dbar
		// 1..6 = duscbt
		// 0 = g

		void keylhapdf(int flavor, string &key); // to return a single flavors
		void keylhapdf(vector<string> &allkeys, bool print); // to return all flavors

		void setlhapdf(string envPath, string fileName, int subset); // initialize with environment variable
		void setlhapdf(string filename, int subs); // initialize with full path and name (overload)

		void getlhapdf(double x, double q, int flavor, double &xpdfx); // to return a single flavor
		void getlhapdf(double x, double q, vector<double> &allxpdfx); // to return all flavors

		double getXmin();
		double getXmax();
		double getQ2min();
		double getQ2max();
		void checkX(double x);
		void checkQ2(double Q2);
		
		void prntlhapdf(double &xpdfx); // to print a single flavor
		void prntlhapdf(vector<double> &allxpdfx); // to print all flavors

		double simpsonTest(int nsegments, double s); // to check if integrate(sum(x*pdf(x))*dx) = 1
	
	private:
		//LHAPDFWrap m_pdf;
		int m_subset;
		vector<string> m_allflavorkey;
		bool m_hasXminWarningAppearead;
		bool m_hasXmaxWarningAppearead;
		bool m_hasQ2minWarningAppearead;
		bool m_hasQ2maxWarningAppearead;
};

}
#endif
