#include "mosesParticleRecord.h"

using namespace std;

namespace Moses1 {
	
mosesParticleRecord::mosesParticleRecord(string name, TFile* file)
{
	initialize(name, file);
}

mosesParticleRecord::~mosesParticleRecord(){}

void mosesParticleRecord::initialize(string name, TFile* file)
{
	m_name = name;

    m_dir = file->mkdir(m_name.c_str());
	m_dir->cd();
    cout << "built the " << m_name << " dir, ";

	m_isHardProcess = new vector<int>;
	m_id   = new vector<int>;
	m_cg   = new vector<double>;
	m_m0   = new vector<double>;
	m_p0   = new vector<double>;
	m_px   = new vector<double>;
	m_py   = new vector<double>;
	m_pz   = new vector<double>;
	m_xVtx = new vector<double>;
	m_yVtx = new vector<double>;
	m_zVtx = new vector<double>;

    string dataTreeName  = name+"Tree";
    string dataTreeTitle = name;
    m_particleDataTree = new TTree(dataTreeName.c_str(),dataTreeTitle.c_str());
	m_particleDataTree->Branch("isHardProcess", &m_isHardProcess);
    m_particleDataTree->Branch("id", &m_id);
	m_particleDataTree->Branch("cg", &m_cg);
    m_particleDataTree->Branch("m0", &m_m0);
    m_particleDataTree->Branch("p0", &m_p0);
    m_particleDataTree->Branch("px", &m_px);
    m_particleDataTree->Branch("py", &m_py);
    m_particleDataTree->Branch("pz", &m_pz);
    m_particleDataTree->Branch("xVtx", &m_xVtx);
    m_particleDataTree->Branch("yVtx", &m_yVtx);
    m_particleDataTree->Branch("zVtx", &m_zVtx);
    cout << "data tree " << endl;;
}

bool mosesParticleRecord::fillRecordTree()
{
	m_particleDataTree->Fill();
	return true;
}

bool mosesParticleRecord::writeRecordTree()
{
	m_dir->cd();
	m_particleDataTree->Write();
	return true;
}

bool mosesParticleRecord::clearParticleRecord()
{
	if(m_isHardProcess!=NULL && m_isHardProcess->size()!=0) m_isHardProcess->clear();
	if(m_id!=NULL && m_id->size()!=0) m_id->clear();
	if(m_cg!=NULL && m_cg->size()!=0) m_cg->clear();
	if(m_m0!=NULL && m_m0->size()!=0) m_m0->clear();
	if(m_p0!=NULL && m_p0->size()!=0) m_p0->clear();
	if(m_px!=NULL && m_px->size()!=0) m_px->clear();
	if(m_py!=NULL && m_py->size()!=0) m_py->clear();
	if(m_pz!=NULL && m_pz->size()!=0) m_pz->clear();
	if(m_xVtx!=NULL && m_xVtx->size()!=0) m_xVtx->clear();
	if(m_yVtx!=NULL && m_yVtx->size()!=0) m_yVtx->clear();
	if(m_zVtx!=NULL && m_zVtx->size()!=0) m_zVtx->clear();
	return true;
}

bool mosesParticleRecord::fillParticleRecord(Particle &particle, int isHardProcess)
{
	m_isHardProcess->push_back( isHardProcess );
	m_id->push_back( particle.id() );
	m_cg->push_back( particle.charge() );
	m_m0->push_back( particle.m() );
	m_p0->push_back( particle.e() );
	m_px->push_back( particle.px() );
	m_py->push_back( particle.py() );
	m_pz->push_back( particle.pz() );
	m_xVtx->push_back( particle.xProd() );
	m_yVtx->push_back( particle.yProd() );
	m_zVtx->push_back( particle.zProd() );
	return true;
}






//-------------------------------------------------------



eventParticles::eventParticles(TTree* particleRecordTree)
{
	m_particleRecordTree = particleRecordTree;

    m_particleRecordTree->SetBranchAddress("isHardProcess",&m_isHardProcess);
    m_particleRecordTree->SetBranchAddress("id",&m_id);
	m_particleRecordTree->SetBranchAddress("cg",&m_cg);
    m_particleRecordTree->SetBranchAddress("m0",&m_m0);
    m_particleRecordTree->SetBranchAddress("p0",&m_p0);
    m_particleRecordTree->SetBranchAddress("px",&m_px);
    m_particleRecordTree->SetBranchAddress("py",&m_py);
    m_particleRecordTree->SetBranchAddress("pz",&m_pz);
    m_particleRecordTree->SetBranchAddress("xVtx",&m_xVtx);
    m_particleRecordTree->SetBranchAddress("yVtx",&m_yVtx);
    m_particleRecordTree->SetBranchAddress("zVtx",&m_zVtx);
	m_size = m_id->size();
}

eventParticles::~eventParticles(){}

void eventParticles::getEntry(int eventEntry)
{
	m_particleRecordTree->GetEntry(eventEntry);
}

unsigned int eventParticles::size()
{
	return m_size;
}

bool eventParticles::isHardProcess(int particleEntry)
{
	return (*m_isHardProcess)[particleEntry];
}

int  eventParticles::id(int particleEntry)
{
	return (*m_id)[particleEntry];
}

double eventParticles::charge(int particleEntry)
{
	return (*m_cg)[particleEntry];
}

double eventParticles::m0(int particleEntry)
{
	return (*m_m0)[particleEntry];
}

double eventParticles::p0(int particleEntry)
{
	return (*m_p0)[particleEntry];
}

double eventParticles::px(int particleEntry)
{
	return (*m_px)[particleEntry];
}

double eventParticles::py(int particleEntry)
{
	return (*m_py)[particleEntry];
}

double eventParticles::pz(int particleEntry)
{
	return (*m_pz)[particleEntry];
}

double eventParticles::xVtx(int particleEntry)
{
	return (*m_xVtx)[particleEntry];
}

double eventParticles::yVtx(int particleEntry)
{
	return (*m_yVtx)[particleEntry];
}

double eventParticles::zVtx(int particleEntry)
{
	return (*m_zVtx)[particleEntry];
}


}
